# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, UbuntuPlugin,
                                SoSPredicate, CosPlugin, PluginOpt,
                                DebianPlugin)


class Docker(Plugin, CosPlugin):

    short_desc = 'Docker containers'
    plugin_name = 'docker'
    profiles = ('container',)

    option_list = [
        PluginOpt('all', default=False,
                  desc='collect for all containers, even terminated ones'),
        PluginOpt('logs', default=False,
                  desc='collect stdout/stderr logs for containers'),
        PluginOpt('size', default=False,
                  desc='collect image sizes for docker ps')
    ]

    def setup(self):
        self.add_copy_spec([
            "/etc/docker/daemon.json",
            "/var/lib/docker/repositories-*"
        ])

        self.add_env_var([
            'DOCKER_BUILD_PROXY',
            'DOCKER_RUN_PROXY'
        ])

        self.add_journal(units="docker")
        self.add_dir_listing('/etc/docker', recursive=True)

        self.set_cmd_predicate(SoSPredicate(self, services=["docker"]))

        subcmds = [
            'events --since 24h --until 1s',
            'ps',
            'stats --no-stream',
            'version',
            'volume ls'
        ]

        for subcmd in subcmds:
            self.add_cmd_output(f"docker {subcmd}")

        self.add_cmd_output("docker info",
                            tags="docker_info")
        self.add_cmd_output("docker images",
                            tags="docker_images")
        self.add_cmd_output("docker ps -a",
                            tags="docker_list_containers")

        # separately grab these separately as they can take a *very* long time
        if self.get_option('size'):
            self.add_cmd_output('docker ps -as', priority=100)
            self.add_cmd_output('docker system df', priority=100)

        nets = self.collect_cmd_output('docker network ls')

        if nets['status'] == 0:
            networks = [n.split()[1] for n in nets['output'].splitlines()[1:]]
            for net in networks:
                self.add_cmd_output(f"docker network inspect {net}")

        containers = [
            c[0] for c in self.get_containers(runtime='docker',
                                              get_all=self.get_option('all'))
        ]
        images = self.get_container_images(runtime='docker')
        volumes = self.get_container_volumes(runtime='docker')

        for container in containers:
            self.add_cmd_output(f"docker inspect {container}",
                                subdir='containers')
            if self.get_option('logs'):
                self.add_cmd_output(f"docker logs -t {container}",
                                    subdir='containers')

        for img in images:
            name, img_id = img
            insp = name if 'none' not in name else img_id
            self.add_cmd_output(f"docker inspect {insp}", subdir='images',
                                tags="docker_image_inspect")
            self.add_cmd_output(
                f"docker image history {insp}",
                subdir='images/history',
                tags='docker_image_tree'
            )

        for vol in volumes:
            self.add_cmd_output(f"docker volume inspect {vol}",
                                subdir='volumes')

    def postproc(self):
        # Attempts to match key=value pairs inside container inspect output
        # for potentially sensitive items like env vars that contain passwords.
        # Typically, these will be seen in env elements or similar, and look
        # like this:
        #             "Env": [
        #                "mypassword=supersecret",
        #                "container=oci"
        #             ],
        # This will mask values when the variable name looks like it may be
        # something worth obfuscating.

        env_regexp = r'(?P<var>(pass|key|secret|PASS|KEY|SECRET).*?)=' \
                      '(?P<value>.*?)"'
        self.do_cmd_output_sub('*inspect*', env_regexp,
                               r'\g<var>=********"')


class RedHatDocker(Docker, RedHatPlugin):

    packages = ('docker', 'docker-latest', 'docker-io', 'docker-engine',
                'docker-ce', 'docker-ee')

    def setup(self):
        super().setup()

        self.add_copy_spec([
            "/etc/udev/rules.d/80-docker.rules",
            "/etc/containers/"
        ])


class UbuntuDocker(Docker, UbuntuPlugin, DebianPlugin):

    packages = ('docker.io', 'docker-engine', 'docker-ce', 'docker-ee')

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/default/docker",
            "/run/docker/libcontainerd/containerd/events.log"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import re

from sos.report.plugins import Plugin, IndependentPlugin


class Salt(Plugin, IndependentPlugin):

    short_desc = 'Salt'

    plugin_name = 'salt'
    profiles = ('sysmgmt',)

    packages = ('salt', 'salt-minion', 'venv-salt-minion', 'salt-common',)

    def setup(self):
        all_logs = self.get_option("all_logs")

        if not all_logs:
            self.add_copy_spec("/var/log/salt/minion")
        else:
            self.add_copy_spec("/var/log/salt")

        self.add_copy_spec([
            "/var/log/venv-salt-minion.log",
            "/var/log/salt-ssh.log",
        ])

        self.add_copy_spec([
            "/etc/salt",
            "/etc/venv-salt-minion/",
            "/usr/local/etc/salt",
        ])
        self.add_forbidden_path([
            "/etc/salt/pki/*/*.pem",
            "/etc/venv-salt-minion/pki/*/*.pem",
            "/usr/local/etc/salt/pki/*/*.pem",
        ])

        self.add_cmd_output([
            "systemctl --full status salt-minion",
            "systemctl --full status venv-salt-minion",
            "salt-minion --versions-report",
            "venv-salt-minion --versions-report",
            "salt-call --local grains.items --out yaml",
            "venv-salt-call --local grains.items --out yaml",
        ], timeout=30)

    def postproc(self):
        regexp = r'(^\s+.*(pass|secret|(?<![A-z])key(?![A-z])).*:\ ).+$'
        subst = r'\1******'
        self.do_path_regex_sub("/etc/salt/*", regexp, subst)

        # Obfuscate grain entries like `password: mypass` or
        # `secret: mysecret`
        grain_regexp = re.compile("(^.*(pass|secret|key).*:)(.*)$",
                                  re.MULTILINE)
        self.do_cmd_output_sub("salt-call", grain_regexp, subst)
        self.do_cmd_output_sub("venv-salt-call", grain_regexp, subst)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin, PluginOpt


class ContainersCommon(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'Common container configs under {/etc,/usr/share}/containers'
    plugin_name = 'containers_common'
    profiles = ('container', )
    packages = ('containers-common', )
    option_list = [
        PluginOpt('rootlessusers', default='', val_type=str,
                  desc='colon-delimited list of users to collect for')
    ]

    def setup(self):
        self.add_copy_spec([
            '/etc/containers/*',
            '/usr/share/containers/*',
            '/etc/subuid',
            '/etc/subgid',
        ])

        self.add_file_tags({
            "/etc/containers/policy.json": "containers_policy"
        })

        users_opt = self.get_option('rootlessusers')
        users_list = []
        if users_opt:
            users_list = [x for x in users_opt.split(':') if x]

        user_subcmds = [
            'podman info',
            'podman unshare cat /proc/self/uid_map',
            'podman unshare cat /proc/self/gid_map',
            'podman images',
            'podman images --digests',
            'podman pod ps',
            'podman port --all',
            'podman ps',
            'podman ps -a',
            'podman stats --no-stream --all',
            'podman version',
            'podman volume ls',
            'buildah info',
            'buildah unshare cat /proc/self/uid_map',
            'buildah unshare cat /proc/self/gid_map',
            'buildah containers',
            'buildah containers --all',
            'buildah images',
            'buildah images --all',
            'buildah version',
        ]
        for user in users_list:
            # collect user's containers' config
            expanded_user = os.path.expanduser(f'~{user}')
            self.add_copy_spec(
                f'{expanded_user}/.config/containers/')
            # collect user-status
            self.add_cmd_output(f'loginctl user-status {user}')
            # collect the user's related commands
            self.add_cmd_output([
                f'machinectl -q shell {user}@ /usr/bin/{cmd}'
                for cmd in user_subcmds
            ], foreground=True)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Sadique Puthen <sputhenp@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenSSL(Plugin):

    short_desc = 'OpenSSL configuration'

    plugin_name = "openssl"
    profiles = ('network', 'security')
    packages = ('openssl',)
    verify_packages = ('openssl.*',)

    def postproc(self):
        protect_keys = [
            "input_password",
            "output_password",
            "challengePassword"
        ]

        regexp = fr"^(\s*#?\s*({'|'.join(protect_keys)}).*=)(.*)"

        self.do_file_sub(
            '/etc/ssl/openssl.cnf',
            regexp,
            r"\1 ******"
        )


class RedHatOpenSSL(OpenSSL, RedHatPlugin):

    files = ('/etc/pki/tls/openssl.cnf',)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/pki/tls/openssl.cnf")


class DebianOpenSSL(OpenSSL, DebianPlugin, UbuntuPlugin):

    files = ('/etc/ssl/openssl.cnf',)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/ssl/openssl.cnf")

# vim: set et ts=4 sw=4 :



# Copyright 2023 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.preppers import SoSPrepper


class UsernamePrepper(SoSPrepper):
    """
    This prepper is used to source usernames from various `last` output content
    as well as a couple select files. This prepper will also leverage the
    --usernames option.
    """

    name = 'username'

    skip_list = [
        'core',
        'nobody',
        'nfsnobody',
        'shutdown',
        'stack',
        'reboot',
        'root',
        'timeout:',
        'ubuntu',
        'username',
        'wtmp'
    ]

    def _get_items_for_username(self, archive):
        items = set()
        _files = [
            'sos_commands/login/lastlog_-u_1000-60000',
            'sos_commands/login/lastlog_-u_60001-65536',
            'sos_commands/login/lastlog_-u_65537-4294967295',
            'sos_commands/login/lastlog2',
            # AD users will be reported here, but favor the lastlog files since
            # those will include local users who have not logged in
            'sos_commands/login/last',
            'sos_commands/login/last_-F',
            'sos_commands/login/lslogins',
            'etc/cron.allow',
            'etc/cron.deny'
        ]
        for _file in _files:
            content = archive.get_file_content(_file)
            if not content:
                continue
            for line in content.splitlines():
                try:
                    user = line.split()[0].lower()
                    if "lslogins" in _file:
                        if int(line.split()[0]) >= 1000:
                            user = line.split()[1].lower()
                        else:
                            continue
                    if user and user not in self.skip_list:
                        items.add(user)
                        if '\\' in user:
                            items.add(user.split('\\')[-1])
                except Exception:
                    # empty line or otherwise unusable for name sourcing
                    pass

        for opt_user in self.opts.usernames:
            if opt_user not in self.skip_list:
                items.add(opt_user)

        return items

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Red Hat, Inc., Eugene Teo <eteo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class SystemTap(Plugin, IndependentPlugin):

    short_desc = 'SystemTap dynamic instrumentation'

    plugin_name = 'systemtap'
    profiles = ('debug', 'performance')

    commands = ('stap',)
    packages = ('systemtap', 'systemtap-runtime')

    def setup(self):
        self.add_cmd_output([
            "stap -V 2",
            "uname -r",
            "stap-report"
        ])

# vim: set et ts=4 sw=4 :



# Copyright Red Hat 2020, Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

# pylint: disable=too-many-branches

import fnmatch
import inspect
import logging
import os
import re

from shlex import quote
from sos.policies import load
from sos.policies.init_systems import InitSystem
from sos.collector.transports.juju import JujuSSH
from sos.collector.transports.control_persist import SSHControlPersist
from sos.collector.transports.local import LocalTransport
from sos.collector.transports.oc import OCTransport
from sos.collector.transports.saltstack import SaltStackMaster
from sos.collector.exceptions import (CommandTimeoutException,
                                      ConnectionException,
                                      UnsupportedHostException,
                                      InvalidTransportException)
from sos.utilities import sos_parse_version

TRANSPORTS = {
    'local': LocalTransport,
    'control_persist': SSHControlPersist,
    'oc': OCTransport,
    'saltstack': SaltStackMaster,
    'juju': JujuSSH,
}


class SosNode():

    def __init__(self, address, commons, password=None, local_sudo=None,
                 load_facts=True):
        self.address = address.strip()
        self.commons = commons
        self.opts = commons['cmdlineopts']
        self._assign_config_opts()
        self.tmpdir = commons['tmpdir']
        self.hostlen = commons['hostlen']
        self.need_sudo = commons['need_sudo']
        self.sos_options = commons['sos_options']
        self.node_config_file = self.opts.node_config_file
        self.inherit_config_file = self.opts.inherit_config_file
        self.local = False
        self.host = None
        self.cluster = None
        self.hostname = None
        self.sos_env_vars = {}
        self._env_vars = {}
        self._password = password or self.opts.password
        if not self.opts.nopasswd_sudo and not self.opts.sudo_pw:
            self.opts.sudo_pw = self._password
        # override local sudo from any other source
        if local_sudo:
            self.opts.sudo_pw = local_sudo
        self.sos_path = None
        self.retrieved = False
        self.hash_retrieved = False
        self.file_list = []
        self.sos_info = {
            'version': None,
            'enabled': [],
            'disabled': [],
            'options': [],
            'presets': [],
            'sos_cmd': commons['sos_cmd']
        }
        self.sos_bin = 'sosreport'
        self.soslog = logging.getLogger('sos')
        self.ui_log = logging.getLogger('sos_ui')
        self._transport = self._load_remote_transport(commons)
        # Overwrite need_sudo if transports default_user
        # is set and is not root.
        if self._transport.default_user:
            self.need_sudo = self._transport.default_user != 'root'
        try:
            self._transport.connect(self._password)
        except Exception as err:
            self.log_error(f'Unable to open remote session: {err}')
            raise
        # load the host policy now, even if we don't want to load further
        # host information. This is necessary if we're running locally on the
        # cluster primary but do not want a local report as we still need to do
        # package checks in that instance
        self.host = self.determine_host_policy()
        self.hostname = self._transport.hostname
        if self.local and self.opts.no_local:
            load_facts = False
        if self.connected and load_facts:
            if not self.host:
                self._transport.disconnect()
                return None
            if self.local:
                if self.check_in_container():
                    self.host.containerized = False
            if self.host.containerized:
                self.create_sos_container()
            self._load_sos_info()
        return None

    @property
    def connected(self):
        if self._transport:
            return self._transport.connected
        # if no transport, we're running locally
        return True

    def disconnect(self):
        """Wrapper to close the remote session via our transport agent
        """
        self._transport.disconnect()

    def _load_remote_transport(self, commons):
        """Determine the type of remote transport to load for this node, then
        return an instantiated instance of that transport
        """
        if self.address in ['localhost', '127.0.0.1']:
            self.local = True
            return LocalTransport(self.address, commons)
        if self.opts.transport in TRANSPORTS:
            return TRANSPORTS[self.opts.transport](self.address, commons)
        if self.opts.transport != 'auto':
            self.log_error(
                "Connection failed: unknown or unsupported transport "
                f"{self.opts.transport}"
            )
            raise InvalidTransportException(self.opts.transport)
        return SSHControlPersist(self.address, commons)

    def _fmt_msg(self, msg):
        return f"{self._hostname:<{self.hostlen + 1}} : {msg}"

    @property
    def env_vars(self):
        if not self._env_vars:
            if self.local:
                self._env_vars = os.environ.copy()
            else:
                ret = self.run_command("env --null")
                if ret['status'] == 0:
                    for ln in ret['output'].split('\x00'):
                        if not ln:
                            continue
                        _val = ln.split('=')
                        self._env_vars[_val[0]] = _val[1]
        return self._env_vars

    def set_node_manifest(self, manifest):
        """Set the manifest section that this node will write to
        """
        self.manifest = manifest
        self.manifest.add_field('hostname', self._hostname)
        self.manifest.add_field('policy', self.host.os_release_name)
        self.manifest.add_field('sos_version', self.sos_info['version'])
        self.manifest.add_field('final_sos_command', '')
        self.manifest.add_field('transport', self._transport.name)

    def check_in_container(self):
        """
        Tries to identify if we are currently running in a container or not.
        """
        if os.path.exists('/run/.containerenv'):
            self.log_debug('Found /run/.containerenv. Running in container.')
            return True
        if os.environ.get('container') is not None:
            self.log_debug("Found env var 'container'. Running in container")
            return True
        return False

    def create_sos_container(self):
        """If the host is containerized, create the container we'll be using
        """
        if self.host.containerized:
            cmd = self.host.create_sos_container(
                image=self.opts.image,
                auth=self.get_container_auth(),
                force_pull=self.opts.force_pull_image
            )
            res = self.run_command(cmd, need_root=True)
            if res['status'] in [0, 125]:
                if res['status'] == 125:
                    if 'unable to retrieve auth token' in res['output']:
                        self.log_error(
                            "Could not pull image. Provide either a username "
                            "and password or authfile"
                        )
                        raise Exception
                    if 'unknown: Not found' in res['output']:
                        self.log_error('Specified image not found on registry')
                        raise Exception
                    # 'name exists' with code 125 means the container was
                    # created successfully, so ignore it.
                # initial creations leads to an exited container, restarting it
                # here will keep it alive for us to exec through
                ret = self.run_command(self.host.restart_sos_container(),
                                       need_root=True)
                if ret['status'] == 0:
                    self.log_info("Temporary container "
                                  f"{self.host.sos_container_name} created")
                    return True
                self.log_error("Could not start container after create: "
                               f"{ret['output']}")
                raise Exception

            self.log_error("Could not create container on host: "
                           f"{res['output']}")
            raise Exception
        return False

    def get_container_auth(self):
        """Determine what the auth string should be to pull the image used to
        deploy our temporary container
        """
        if self.opts.registry_user:
            return self.host.runtimes['default'].fmt_registry_credentials(
                self.opts.registry_user,
                self.opts.registry_password
            )
        return self.host.runtimes['default'].fmt_registry_authfile(
            self.opts.registry_authfile or self.host.container_authfile
        )

    def file_exists(self, fname, need_root=False):
        """Checks for the presence of fname on the remote node"""
        try:
            res = self.run_command(f"stat {fname}", need_root=need_root)
            return res['status'] == 0
        except Exception:
            return False

    @property
    def _hostname(self):
        if self.hostname and 'localhost' not in self.hostname:
            return self.hostname
        return self.address

    def _sanitize_log_msg(self, msg):
        """Attempts to obfuscate sensitive information in log messages such as
        passwords"""
        reg = r'(?P<var>(pass|key|secret|PASS|KEY|SECRET).*?=)(?P<value>.*?\s)'
        return re.sub(reg, r'\g<var>****** ', msg)

    def ui_msg(self, msg):
        """Format a ui message that includes host name and formatting"""
        self.ui_log.info(self._fmt_msg(msg))

    def log_info(self, msg):
        """Used to print and log info messages"""
        caller = inspect.stack()[1][3]
        lmsg = f'[{self._hostname}:{caller}] {msg}'
        self.soslog.info(lmsg)

    def log_error(self, msg):
        """Used to print and log error messages"""
        caller = inspect.stack()[1][3]
        lmsg = f'[{self._hostname}:{caller}] {msg}'
        self.soslog.error(lmsg)

    def log_debug(self, msg):
        """Used to print and log debug messages"""
        msg = self._sanitize_log_msg(msg)
        caller = inspect.stack()[1][3]
        msg = f'[{self._hostname}:{caller}] {msg}'
        self.soslog.debug(msg)

    def _format_cmd(self, cmd):
        """If we need to provide a sudo or root password to a command, then
        here we prefix the command with the correct bits
        """
        is_root = self._env_vars.get("USER", os.environ.get("USER")) == 'root'
        if self.local and is_root:
            return cmd
        if self.opts.become_root:
            return f"su -c {quote(cmd)}"
        if self.need_sudo:
            return f"sudo -S {cmd}"
        return cmd

    def _load_sos_info(self):
        """Queries the node for information about the installed version of sos
        """
        ver = None
        rel = None
        if self.host.container_version_command is None:
            pkg = self.host.package_manager.pkg_version(self.host.sos_pkg_name)
            if pkg is not None:
                ver = '.'.join(pkg['version'])
                if pkg['release']:
                    rel = pkg['release']

        else:
            # use the containerized policy's command
            pkgs = self.run_command(self.host.container_version_command,
                                    use_container=True, need_root=True)
            if pkgs['status'] == 0:
                _, ver, rel = pkgs['output'].strip().split('-')

        if ver:
            if len(ver.split('.')) == 2:
                # safeguard against maintenance releases throwing off the
                # comparison by parse_version
                ver += '.0'
            try:
                ver += f'-{rel.split(".")[0]}'
            except Exception as err:
                self.log_debug(f"Unable to fully parse sos release: {err}")

        self.sos_info['version'] = ver

        if self.sos_info['version']:
            self.log_info(f'sos version is {self.sos_info["version"]}')
        else:
            if not self.address == self.opts.primary:
                # in the case where the 'primary' enumerates nodes but is not
                # intended for collection (bastions), don't worry about sos not
                # being present
                self.log_error('sos is not installed on this node')
            self.connected = False
            return False
        # sos-4.0 changes the binary
        if self.check_sos_version('4.0'):
            self.sos_bin = 'sos report'
        cmd = f"{self.sos_bin} -l"
        sosinfo = self.run_command(cmd, use_container=True, need_root=True)
        if sosinfo['status'] == 0:
            self._load_sos_plugins(sosinfo['output'])
        if self.check_sos_version('3.6'):
            self._load_sos_presets()
        return None

    def _load_sos_presets(self):
        cmd = f'{self.sos_bin} --list-presets'
        res = self.run_command(cmd, use_container=True, need_root=True)
        if res['status'] == 0:
            for line in res['output'].splitlines():
                if line.strip().startswith('name:'):
                    pname = line.split('name:')[1].strip()
                    self.sos_info['presets'].append(pname)

    def _load_sos_plugins(self, sosinfo):
        ENABLED = 'The following plugins are currently enabled:'
        DISABLED = 'The following plugins are currently disabled:'
        ALL_OPTIONS = 'The following options are available for ALL plugins:'
        OPTIONS = 'The following plugin options are available:'
        PROFILES = 'Profiles:'

        enablereg = ENABLED + '(.*?)' + DISABLED
        disreg = DISABLED + '(.*?)' + ALL_OPTIONS
        optreg = OPTIONS + '(.*?)' + PROFILES
        proreg = PROFILES + '(.*?)' + '\n\n'

        self.sos_info['enabled'] = self._regex_sos_help(enablereg, sosinfo)
        self.sos_info['disabled'] = self._regex_sos_help(disreg, sosinfo)
        self.sos_info['options'] = self._regex_sos_help(optreg, sosinfo)
        self.sos_info['profiles'] = self._regex_sos_help(proreg, sosinfo, True)

    def _regex_sos_help(self, regex, sosinfo, is_list=False):
        res = []
        for result in re.findall(regex, sosinfo, re.S):
            for line in result.splitlines():
                if not is_list:
                    try:
                        if ls := line.split():
                            res.append(ls[0])
                    except Exception as err:
                        self.log_debug(f"Error parsing sos help: {err}")
                else:
                    r = line.split(',')
                    res.extend(p.strip() for p in r if p.strip())
        return res

    def read_file(self, to_read):
        """Reads the specified file and returns the contents"""
        try:
            self.log_info(f"Reading file {to_read}")
            return self._transport.read_file(to_read)
        except Exception as err:
            self.log_error(f"Exception while reading {to_read}: {err}")
            return ''

    def determine_host_policy(self):
        """Attempts to identify the host installation against supported
        distributions
        """
        if self.local:
            self.log_info(
                f"using local policy {self.commons['policy'].os_release_name}")
            return self.commons['policy']
        host = load(cache={}, sysroot=self.opts.sysroot, init=InitSystem(),
                    probe_runtime=True,
                    remote_exec=self._transport.run_command,
                    remote_check=self.read_file('/etc/os-release'))
        if host:
            self.log_info(f"loaded policy {host.os_release_name} for host")
            return host
        self.log_error('Unable to determine host installation. Ignoring node')
        raise UnsupportedHostException

    def check_sos_version(self, ver):
        """Checks to see if the sos installation on the node is AT LEAST the
        given ver. This means that if the installed version is greater than
        ver, this will still return True

        :param ver: Version number we are trying to verify is installed
        :type ver:  ``str``

        :returns:   True if installed version is at least ``ver``, else False
        :rtype:     ``bool``
        """
        try:
            _node_ver = self.sos_info['version']
            return sos_parse_version(_node_ver) >= sos_parse_version(ver)
        except Exception as err:
            self.log_error(f"Error checking sos version: {err}")
            return False

    def is_installed(self, pkg):
        """Checks if a given package is installed on the node"""
        if not self.host:
            return False
        return self.host.package_manager.pkg_by_name(pkg) is not None

    def run_command(self, cmd, timeout=180, use_shell='auto', need_root=False,
                    use_container=False, env=None):
        """Runs a given cmd, either via the SSH session or locally

        :param cmd:     The full command to be run
        :type cmd:      ``str``

        :param timeout: Time in seconds to wait for `cmd` to complete
        :type timeout:  ``int``

        :param use_shell: If a shell is needed to run `cmd`, set to True
        :type use_shell:  ``bool`` or ``auto`` for transport-determined

        :param use_container: Run this command in a container *IF* the host
                              is a containerized host
        :type use_container: ``bool``

        :param env: Pass environment variables to set for this `cmd`
        :type env:  ``dict``
        """
        if not self.connected and not self.local:
            self.log_debug('Node is disconnected, attempting to reconnect')
            try:
                reconnected = self._transport.reconnect(self._password)
                if not reconnected:
                    self.log_debug('Failed to reconnect to node')
                    raise ConnectionException
            except Exception as err:
                self.log_debug(f"Error while trying to reconnect: {err}")
                raise
        if use_container and self.host.containerized:
            cmd = self.host.format_container_command(cmd)
        if need_root:
            cmd = self._format_cmd(cmd)
        if env:
            _cmd_env = self.env_vars
            env = _cmd_env.update(env)
        return self._transport.run_command(cmd, timeout, need_root, env,
                                           use_shell)

    def sosreport(self):
        """Run an sos report on the node, then collect it"""
        try:
            path = self.execute_sos_command()
            if path:
                self.finalize_sos_path(path)
            else:
                self.log_error('Unable to determine path of sos archive')
            if self.sos_path:
                self.retrieved = self.retrieve_sosreport()
        except Exception as err:
            self.log_error(f"Error during sos execution: {err}")
        self.cleanup()

    def _preset_exists(self, preset):
        """Verifies if the given preset exists on the node"""
        return preset in self.sos_info['presets']

    def _plugin_exists(self, plugin):
        """Verifies if the given plugin exists on the node"""
        return any(plugin in s for s in [self.sos_info['enabled'],
                                         self.sos_info['disabled']])

    def _check_enabled(self, plugin):
        """Checks to see if the plugin is default enabled on node"""
        return plugin in self.sos_info['enabled']

    def _check_disabled(self, plugin):
        """Checks to see if the plugin is default disabled on node"""
        return plugin in self.sos_info['disabled']

    def _plugin_option_exists(self, opt):
        """Attempts to verify that the given option is available on the node.
        Note that we only get available options for enabled plugins, so if a
        plugin has been force-enabled we cannot validate if the plugin option
        is correct or not"""
        plug = opt.split('.')[0]
        if not self._plugin_exists(plug):
            return False
        if (self._check_disabled(plug) and
                plug not in self.opts.enable_plugins):
            return False
        if self._check_enabled(plug):
            return opt in self.sos_info['options']
        # plugin exists, but is normally disabled. Assume user knows option is
        # valid when enabling the plugin
        return True

    def _fmt_sos_opt_list(self, opts):
        """Returns a comma delimited list for sos plugins that are confirmed
        to exist on the node"""
        return ','.join(o for o in opts if self._plugin_exists(o))

    def set_cluster(self, cluster):
        """Expose the node to the cluster profile determined for the
        environment
        """
        self.cluster = cluster

    def update_cmd_from_cluster(self):
        """This is used to modify the sos report command run on the nodes.
        By default, sos report is run without any options, using this will
        allow the profile to specify what plugins to run or not and what
        options to use.

        This will NOT override user supplied options.
        """
        if self.cluster.sos_preset:
            if not self.preset:
                self.preset = self.cluster.sos_preset
            else:
                self.log_info('Cluster specified preset '
                              f'{self.cluster.sos_preset} but user has also '
                              'defined a preset. Using user specification.')
        if self.cluster.sos_plugins:
            for plug in self.cluster.sos_plugins:
                if plug not in self.enable_plugins:
                    self.enable_plugins.append(plug)

        if self.cluster.sos_options:
            for opt in self.cluster.sos_options:
                # take the user specification over any cluster defaults
                if opt not in self.sos_options:
                    self.sos_options[opt] = self.cluster.sos_options[opt]

        if self.cluster.sos_plugin_options:
            for opt in self.cluster.sos_plugin_options:
                if not any(opt in o for o in self.plugopts):
                    option = f'{opt}={self.cluster.sos_plugin_options[opt]}'
                    self.plugopts.append(option)

        # set primary-only options
        if self.cluster.check_node_is_primary(self):
            with self.cluster.lock:
                self.cluster.set_primary_options(self)
        else:
            with self.cluster.lock:
                self.cluster.set_node_options(self)

    def _assign_config_opts(self):
        """From the global opts configuration, assign those values locally
        to this node so that they may be acted on individually.
        """
        # assign these to new, private copies
        self.only_plugins = list(self.opts.only_plugins)
        self.skip_plugins = list(self.opts.skip_plugins)
        self.enable_plugins = list(self.opts.enable_plugins)
        self.plugopts = list(self.opts.plugopts)
        self.preset = list(self.opts.preset)

    def finalize_sos_cmd(self):
        """Use host facts and compare to the cluster type to modify the sos
        command if needed"""
        sos_cmd = self.sos_info['sos_cmd']
        label = self.determine_sos_label()
        if label:
            sos_cmd = f'{sos_cmd} {quote(label)} '

        sos_opts = []

        # sos-3.6 added --threads
        if self.check_sos_version('3.6'):
            # 4 threads is the project's default
            if self.opts.threads != 4:
                sos_opts.append(f'--threads={quote(str(self.opts.threads))}')

        # sos-3.7 added options
        if self.check_sos_version('3.7'):
            if self.opts.plugin_timeout:
                sos_opts.append(
                    f'--plugin-timeout={quote(str(self.opts.plugin_timeout))}')

        # sos-3.8 added options
        if self.check_sos_version('3.8'):
            if self.opts.allow_system_changes:
                sos_opts.append('--allow-system-changes')

            if self.opts.no_env_vars:
                sos_opts.append('--no-env-vars')

            if self.opts.since:
                sos_opts.append(f'--since={quote(self.opts.since)}')

        if self.check_sos_version('4.1'):
            if self.opts.skip_commands:
                sos_opts.append('--skip-commands='
                                f'{quote(",".join(self.opts.skip_commands))}')
            if self.opts.skip_files:
                sos_opts.append(
                    f'--skip-files={quote(",".join(self.opts.skip_files))}'
                )

        if self.check_sos_version('4.2'):
            if self.opts.cmd_timeout:
                sos_opts.append(
                    f'--cmd-timeout={quote(str(self.opts.cmd_timeout))}'
                )

        # handle downstream versions that backported this option
        if self.check_sos_version('4.3') or self.check_sos_version('4.2-13'):
            if self.opts.container_runtime != 'auto':
                sos_opts.append(
                    f"--container-runtime={self.opts.container_runtime}"
                )
            if self.opts.namespaces:
                sos_opts.append(
                    f"--namespaces={self.opts.namespaces}"
                )

        if self.check_sos_version('4.5.2'):
            if self.opts.journal_size:
                sos_opts.append(f"--journal-size={self.opts.journal_size}")
            if self.opts.low_priority:
                sos_opts.append('--low-priority')

        self.update_cmd_from_cluster()

        sos_cmd = sos_cmd.replace(
            'sosreport',
            os.path.join(self.host.sos_bin_path, self.sos_bin)
        )

        for opt in self.sos_options:
            _val = self.sos_options[opt]
            sos_opts.append(f"--{opt} {_val if _val else ''}")

        if self.plugopts:
            opts = [o for o in self.plugopts
                    if self._plugin_exists(o.split('.')[0])
                    and self._plugin_option_exists(o.split('=')[0])]
            if opts:
                sos_opts.append(f'-k {quote(",".join(o for o in opts))}')

        if self.preset:
            if self._preset_exists(self.preset):
                sos_opts.append(f'--preset={quote(self.preset)}')
            else:
                self.log_debug(f'Requested to enable preset {self.preset} but '
                               'preset does not exist on node')

        if self.only_plugins:
            plugs = [o for o in self.only_plugins if self._plugin_exists(o)]
            if len(plugs) != len(self.only_plugins):
                not_only = list(set(self.only_plugins) - set(plugs))
                self.log_debug(f'Requested plugins {not_only} were requested '
                               'to be enabled but do not exist')
            only = self._fmt_sos_opt_list(self.only_plugins)
            if only:
                sos_opts.append(f'--only-plugins={quote(only)}')
            self.sos_cmd = f"{sos_cmd} {' '.join(sos_opts)}"
            self.log_info(f'Final sos command set to {self.sos_cmd}')
            self.manifest.add_field('final_sos_command', self.sos_cmd)
            return

        if self.skip_plugins:
            # only run skip-plugins for plugins that are enabled
            skip = [o for o in self.skip_plugins if self._check_enabled(o)]
            if len(skip) != len(self.skip_plugins):
                not_skip = list(set(self.skip_plugins) - set(skip))
                self.log_debug(f'Requested to skip plugins {not_skip}, but '
                               'plugins are already not enabled')
            skipln = self._fmt_sos_opt_list(skip)
            if skipln:
                sos_opts.append(f'--skip-plugins={quote(skipln)}')

        if self.enable_plugins:
            # only run enable for plugins that are disabled
            opts = [o for o in self.enable_plugins
                    if o not in self.skip_plugins
                    and self._check_disabled(o) and self._plugin_exists(o)]
            if len(opts) != len(self.enable_plugins):
                not_on = list(set(self.enable_plugins) - set(opts))
                self.log_debug(f'Requested to enable plugins {not_on}, but '
                               'plugins are already enabled or do not exist')
            enable = self._fmt_sos_opt_list(opts)
            if enable:
                sos_opts.append(f'--enable-plugins={quote(enable)}')

        self.sos_cmd = f"{sos_cmd} {' '.join(sos_opts)}"
        self.log_info(f'Final sos command set to {self.sos_cmd}')
        self.manifest.add_field('final_sos_command', self.sos_cmd)

    def determine_sos_label(self):
        """Determine what, if any, label should be added to the sos report"""
        label = ''
        label += self.cluster.get_node_label(self)

        if self.opts.label:
            label += (f'{self.opts.label}' if not label
                      else f'-{self.opts.label}')

        if not label:
            return None

        self.log_debug(f'Label for sos report set to {label}')
        if self.check_sos_version('3.6'):
            lcmd = '--label'
        else:
            lcmd = '--name'
            label = f'{self.address.split(".")[0]}-{label}'
        return f'{lcmd}={label}'

    def finalize_sos_path(self, path):
        """Use host facts to determine if we need to change the sos path
        we are retrieving from"""
        pstrip = self.host.sos_path_strip
        if pstrip:
            path = path.replace(pstrip, '')
        path = path.split()[0]
        self.log_info(f'Final sos path: {path}')
        self.sos_path = path
        self.archive = path.split('/')[-1]
        self.manifest.add_field('collected_archive', self.archive)

    def determine_sos_error(self, rc, stdout):
        if rc == -1:
            return 'sos report process received SIGKILL on node'
        if rc == 1:
            if 'sudo' in stdout:
                return 'sudo attempt failed'
        if rc == 127:
            return 'sos report terminated unexpectedly. Check disk space'
        if len(stdout) > 0:
            return stdout.split('\n')[0:1]
        return f'sos exited with code {rc}'

    def execute_sos_command(self):
        """Run sos report and capture the resulting file path"""
        self.ui_msg('Generating sos report...')
        try:
            path = False
            checksum = False
            config_file_arg = ''
            if self.opts.node_config_file:
                config_file_arg = f'--config-file={self.opts.node_config_file}'
            elif self.opts.inherit_config_file:
                if not self.local:
                    remote_config = f"/tmp/{self.tmpdir.split('/')[-1]}.conf"
                    self._transport.copy_file_to_remote(
                        self.opts.config_file,
                        remote_config)
                    config_file_arg = f'--config-file={remote_config}'
                else:
                    config_file_arg = (
                        f'--config-file={self.opts.config_file}')
            if config_file_arg:
                self.sos_cmd = f"{self.sos_cmd} {config_file_arg}"
            res = self.run_command(self.sos_cmd,
                                   timeout=self.opts.timeout,
                                   use_shell=True,
                                   need_root=True,
                                   use_container=True,
                                   env=self.sos_env_vars)
            if res['status'] == 0:
                for line in res['output'].splitlines():
                    if fnmatch.fnmatch(line, '*sosreport-*tar*'):
                        path = line.strip()
                    if line.startswith((" sha256\t", " md5\t")):
                        checksum = line.split("\t")[1]
                    elif line.startswith("The checksum is: "):
                        checksum = line.split()[3]

                if checksum:
                    self.manifest.add_field('checksum', checksum)
                    if len(checksum) == 32:
                        self.manifest.add_field('checksum_type', 'md5')
                    elif len(checksum) == 64:
                        self.manifest.add_field('checksum_type', 'sha256')
                    else:
                        self.manifest.add_field('checksum_type', 'unknown')
                else:
                    self.manifest.add_field('checksum_type', 'unknown')
            else:
                err = self.determine_sos_error(res['status'], res['output'])
                self.log_debug("Error running sos report. rc = "
                               f"{res['status']} msg = {res['output']}")
                raise Exception(err)
            return path
        except CommandTimeoutException:
            self.log_error('Timeout exceeded')
            raise
        except Exception as err:
            self.log_info(f"Exception during sos report execution: {err}")
            self.ui_msg(f"Error running sos report: {err}")
            raise

    def retrieve_file(self, path):
        """Copies the specified file from the host to our temp dir"""
        destdir = self.tmpdir + '/'
        dest = os.path.join(destdir, path.split('/')[-1])
        try:
            if self.file_exists(path):
                self.log_info(f"Copying remote {path} to local {destdir}")
                return self._transport.retrieve_file(path, dest)
            self.log_debug(f"Attempting to copy remote file {path}, but it"
                           " does not exist on filesystem")
            return False
        except Exception as err:
            self.log_debug(f"Failed to retrieve {path}: {err}")
            return False

    def remove_file(self, path):
        """Removes the spciefied file from the host. This should only be used
        after we have retrieved the file already
        """
        path = ''.join(path.split())
        try:
            if len(path.split('/')) <= 2:  # ensure we have a non '/' path
                self.log_debug(f"Refusing to remove path {path}: appears to "
                               "be incorrect and possibly dangerous")
                return False
            if self.file_exists(path):
                self.log_info(f"Removing file {path}")
                cmd = f"rm -f {path}"
                res = self.run_command(cmd, need_root=True)
                return res['status'] == 0
            self.log_debug(f"Attempting to remove remote file {path}, but "
                           "it does not exist on filesystem")
            return False
        except Exception as e:
            self.log_debug(f'Failed to remove {path}: {e}')
            return False

    def retrieve_sosreport(self):
        """Collect the sos report archive from the node"""
        if self.need_sudo or self.opts.become_root:
            try:
                self.make_archive_readable(self.sos_path)
            except Exception:
                self.log_error('Failed to make archive readable')
                return False
        self.log_info(f'Retrieving sos report from {self.address}')
        self.ui_msg('Retrieving sos report...')
        try:
            ret = self.retrieve_file(self.sos_path)
        except Exception as err:
            self.log_error(err)
            return False
        if ret:
            self.ui_msg('Successfully collected sos report')
            self.file_list.append(self.sos_path.split('/')[-1])
            return True
        self.ui_msg('Failed to retrieve sos report')
        return False

    def remove_sos_archive(self):
        """Remove the sos report archive from the node, since we have
        collected it and it would be wasted space otherwise"""
        if self.sos_path is None or self.local:
            # local transport moves the archive rather than copies it, so there
            # is no archive at the original location to remove
            return
        if 'sosreport' not in self.sos_path:
            self.log_debug(f"Node sos report path {self.sos_path} looks "
                           "incorrect. Not attempting to remove path")
            return
        removed = self.remove_file(self.sos_path)
        if not removed:
            self.log_error('Failed to remove sos report')

    def cleanup(self):
        """Remove the sos archive from the node once we have it locally"""
        self.remove_sos_archive()
        if self.sos_path:
            for ext in ['.sha256', '.md5']:
                if self.remove_file(self.sos_path + ext):
                    break
        cleanup = self.host.set_cleanup_cmd()
        if cleanup:
            self.run_command(cleanup, need_root=True)

    def collect_extra_cmd(self, filenames):
        """Collect the file created by a cluster outside of sos"""
        for filename in filenames:
            try:
                if self.need_sudo or self.opts.become_root:
                    try:
                        self.make_archive_readable(filename)
                    except Exception as err:
                        self.log_error(f"Unable to retrieve file {filename}")
                        self.log_debug(f"Failed to make file {filename} "
                                       f"readable: {err}")
                        continue
                ret = self.retrieve_file(filename)
                if ret:
                    self.file_list.append(filename.split('/')[-1])
                    self.remove_file(filename)
                else:
                    self.log_error(f"Unable to retrieve file {filename}")
            except Exception as e:
                msg = f'Error collecting additional data from primary: {e}'
                self.log_error(msg)

    def make_archive_readable(self, filepath):
        """Used to make the given archive world-readable, which is slightly
        better than changing the ownership outright.

        This is only used when we're not connecting as root.
        """
        cmd = f'chmod o+r {filepath}'
        res = self.run_command(cmd, timeout=10, need_root=True)
        if res['status'] == 0:
            return True
        msg = "Exception while making %s readable. Return code was %s"
        self.log_error(msg % (filepath, res['status']))
        raise Exception

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc., Pratik Bandarkar <pbandark@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Grafana(Plugin, IndependentPlugin):

    short_desc = 'Fetch Grafana configuration, logs and CLI output'
    plugin_name = "grafana"
    profiles = ('services', 'openstack', 'openstack_controller')

    packages = ('grafana',)

    def setup(self):
        if self.is_snap:
            grafana_cli = "grafana.grafana-cli"
            log_path = "/var/snap/grafana/common/data/log/"
            config_path = "/var/snap/grafana/current/conf/grafana.ini"

            self.add_cmd_output("snap info grafana", snap_cmd=True)
        else:
            grafana_cli = "grafana-cli"
            log_path = "/var/log/grafana/"
            config_path = "/etc/grafana/"

        self.add_cmd_output([
            f'{grafana_cli} plugins ls',
            f'{grafana_cli} plugins list-remote',
            f'{grafana_cli} -v',
            'grafana-server -v',
        ], snap_cmd=self.is_snap)

        log_file_pattern = "*.log*" if self.get_option("all_logs") else "*.log"

        self.add_copy_spec([
            log_path + log_file_pattern,
            config_path,
            "/etc/sysconfig/grafana-server",
        ])

    def postproc(self):
        protect_keys = [
            "admin_password",
            "secret_key",
            "password",
            "client_secret",
        ]
        inifile = (
            "/var/snap/grafana/current/conf/grafana.ini"
            if self.is_snap
            else "/etc/grafana/grafana.ini"
        )

        regexp = fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)"
        self.do_path_regex_sub(inifile, regexp, r"\1*********")



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Pci(Plugin, IndependentPlugin):

    short_desc = 'PCI devices'

    plugin_name = "pci"
    profiles = ('hardware', 'system')

    def check_for_bus_devices(self):
        """ Check whether PCI devices present """
        if not self.path_isdir('/proc/bus/pci'):
            return False
        # ensure that more than just the 'devices' file, which can be empty,
        # exists in the pci directory. This implies actual devices are present
        content = self.listdir('/proc/bus/pci')
        if 'devices' in content:
            content.remove('devices')
        return len(content) > 0

    def setup(self):
        self.add_copy_spec([
            "/proc/ioports",
            "/proc/iomem",
            "/proc/bus/pci"
        ])

        if self.check_for_bus_devices():
            self.add_cmd_output("lspci -nnvv", root_symlink="lspci",
                                tags="lspci")
            self.add_cmd_output("lspci -tv")

# vim: set et ts=4 sw=4 :



# Copyright (c) 2012 Adam Stokes <adam.stokes@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, DebianPlugin


class Apparmor(Plugin, DebianPlugin):

    short_desc = 'Apparmor mandatory access control'

    plugin_name = 'apparmor'
    profiles = ('security',)

    def setup(self):
        self.add_copy_spec([
            "/etc/apparmor*"
        ])

        self.add_forbidden_path([
            "/etc/apparmor.d/cache",
            "/etc/apparmor.d/libvirt/libvirt*",
            "/etc/apparmor.d/abstractions"
        ])

        self.add_cmd_output("apparmor_status")

        self.add_dir_listing([
            '/etc/apparmor.d/abstractions',
            '/etc/apparmor.d/libvirt'
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
import os

from sos_tests import StageOneReportTest


class ProcessPluginTest(StageOneReportTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = '-o process -k process.numprocs=100'

    def test_proc_files_collected(self):
        self.assertFileGlobInArchive('/proc/*/status')
        self.assertFileGlobInArchive('/proc/*/stack')
        self.assertFileGlobInArchive('/proc/*/oom_*')

    def test_option_limited_proc_collection(self):
        count = glob.glob(os.path.join(self.archive_path, 'proc/*/status'))
        self.assertTrue(len(count) < 101)



# Copyright (C) 2024 Canonical Ltd., Arif Ali <arif.ali@canonical.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class AptConfScrubTest(StageTwoReportTest):
    """Ensure that sources.list and apt conf are picked up and properly
    scrubbed

    :avocado: tags=stagetwo
    """

    sos_cmd = '-o apt'
    debian_only = True
    files = [
        ('apt-proxy.conf', '/etc/apt/apt.conf.d/50-apt-proxy'),
        ('apt-sources.list', '/etc/apt/sources.list'),
        ('apt-sources.sources', '/etc/apt/sources.list.d/ubuntu.sources'),
    ]

    def test_apt_sources_and_apt_confs_collected(self):
        self.assertFileCollected('/etc/apt/apt.conf.d/50-apt-proxy')
        self.assertFileCollected('/etc/apt/sources.list')
        self.assertFileCollected('/etc/apt/ubuntu.sources')

    def test_apt_sources_and_proxy_scrubbed(self):
        # Ensure that we scrubbed all passwords
        files_to_check = [
            '/etc/apt/apt.conf.d/50-apt-proxy',
            '/etc/apt/sources.list',
            '/etc/apt/sources.list.d/ubuntu.sources',
        ]
        password = 'somesecretpassword'
        for file in files_to_check:
            self.assertFileNotHasContent(file, password)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class RedHatInsights(Plugin, RedHatPlugin):
    """Plugin to capture configuration and logging for the Red Hat Insights
    client. Insights is used to provide ongoing analysis of systems for
    proactive problem mitigation, with the client being one of the primary
    sources of data for the service.

    This plugin will capture configuration information under
    /etc/insighits-client, as well as logs from /var/log/insights-client. A
    single connection test via the `insights-client` command is performed and
    recorded as well for troubleshooting purposes.
    """

    short_desc = 'Red Hat Insights configuration and client'
    plugin_name = 'insights'
    packages = ('redhat-access-insights', 'insights-client')
    profiles = ('system', 'sysmgmt')
    config_and_status = (
        '/etc/insights-client/insights-client.conf',
        '/etc/insights-client/.registered',
        '/etc/insights-client/tags.yaml',
        '/etc/insights-client/malware-detection-config.yml',
        '/etc/redhat-access-insights/redhat-access-insights.conf',
        '/etc/insights-client/.lastupload',
        '/etc/insights-client/machine-id',
    )

    def setup(self):
        self.add_copy_spec(self.config_and_status)
        self.add_copy_spec('/var/lib/insights')

        # Legacy log file location
        self.add_copy_spec("/var/log/redhat-access-insights/*.log")

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/insights-client/*.log*")
        else:
            self.add_copy_spec("/var/log/insights-client/insights-client.log")

        self.add_cmd_output(
            "insights-client --test-connection --net-debug",
            timeout=30
        )

        self.add_dir_listing(["/etc/rhsm", "/sys/kernel", "/var/lib/sss"],
                             recursive=True)

    def postproc(self):
        for conf in self.config_and_status:
            self.do_file_sub(
                conf, r'(password[\t\ ]*=[\t\ ]*)(.+)', r'\1********'
            )

            self.do_file_sub(
                conf, r'(proxy[\t\ ]*=.*)(:)(.*)(@.*)', r'\1\2********\4'
            )
        self.do_paths_http_sub(["/var/log/insights-client/*"])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)


class Services(Plugin):

    short_desc = 'System services'

    plugin_name = "services"
    profiles = ('system', 'boot')

    option_list = [
        PluginOpt('servicestatus', default=False,
                  desc='collect status of all running services')
    ]

    def setup(self):
        self.add_copy_spec([
            "/etc/inittab",
            "/etc/rc.d",
            "/etc/rc.local"
        ])
        if self.get_option('servicestatus'):
            self.add_cmd_output("service --status-all")
        self.add_cmd_output("/sbin/runlevel")

        self.add_dir_listing('/var/lock/subsys')


class RedHatServices(Services, RedHatPlugin):

    def setup(self):
        super().setup()
        self.add_cmd_output("chkconfig --list", root_symlink="chkconfig",
                            tags="chkconfig")


class DebianServices(Services, DebianPlugin, UbuntuPlugin):

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/rc*.d")

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.keyword_map import SoSKeywordMap


class SoSKeywordParser(SoSCleanerParser):
    """Handles parsing for user provided keywords
    """

    name = 'Keyword Parser'
    map_file_key = 'keyword_map'

    def __init__(self, config, workdir, skip_cleaning_files=[]):
        self.mapping = SoSKeywordMap(workdir)
        super().__init__(config, skip_cleaning_files)

    def _parse_line(self, line):
        return line, 0



# Copyright (C) 2018 Red Hat, K.K., Takayoshi Tanaka <tatanaka@redhat.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class MsSQL(Plugin, RedHatPlugin):

    short_desc = 'Microsoft SQL Server on Linux'

    plugin_name = "mssql"
    profiles = ('services',)
    packages = ('mssql-server',)

    option_list = [
        PluginOpt('mssql-conf', default='/var/opt/mssql/mssql.conf',
                  desc='SQL server configuration file')
    ]

    def setup(self):
        mssql_conf = self.get_option('mssql-conf')

        # Pick error log file from mssql_conf.
        # Expecting the following format
        # ```
        # [filelocation]
        # errorlogfile = /var/opt/mssql/log
        # [sqlagent]
        # errorlogfile = /var/opt/mssql/log/sqlagentstartup.log
        # [network]
        # kerberoskeytabfile = /var/opt/mssql/secrets/mssql.keytab
        # ```
        section = ''
        # default values
        errorlogfile = '/var/opt/mssql/log'
        sqlagent_errorlogfile = '/var/opt/mssql/log/sqlagentstartup.log'
        kerberoskeytabfile = None
        try:
            with open(mssql_conf, 'r', encoding='UTF-8') as mfile:
                for line in mfile.read().splitlines():
                    if line.startswith('['):
                        section = line
                        continue
                    words = line.split('=')
                    if words[0].strip() == 'errorlogfile':
                        if section == '[filelocation]':
                            errorlogfile = words[1].strip()
                        elif section == '[sqlagent]':
                            sqlagent_errorlogfile = words[1].strip()
                    elif (words[0].strip() == 'kerberoskeytabfile') and \
                         (section == '[network]'):
                        kerberoskeytabfile = words[1].strip()
        except IOError as ex:
            self._log_error(f'Could not open conf file {mssql_conf}: {ex}')
            return

        # Collect AD authentication configuratoin
        keytab_err = ('keytab file is specfieid in mssql_conf'
                      f' but not found in {kerberoskeytabfile}')
        if kerberoskeytabfile is not None:
            if self.path_isfile(kerberoskeytabfile):
                self.add_dir_listing(kerberoskeytabfile)
                self.add_cmd_output(f'klist -e -k {kerberoskeytabfile}')
            else:
                self._log_error(keytab_err)

        # Expecting mssql_conf doesn't includeno sensitive information.
        self.add_copy_spec([
            mssql_conf,
            errorlogfile + '/*',
            sqlagent_errorlogfile
        ])

        if not self.get_option('all_logs'):
            self.add_copy_spec(errorlogfile + '/*')
            self.add_copy_spec(sqlagent_errorlogfile)
        else:
            self.add_copy_spec(errorlogfile + '/*')
            self.add_copy_spec(sqlagent_errorlogfile)

        self.add_journal(units=['mssql-server'])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc., Sachin Patil <psachin@redhat.com>
# Copyright (C) 2017 Red Hat, Inc., Martin Schuppert <mschuppert@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Gnocchi(Plugin):

    short_desc = 'Gnocchi - Metric as a service'
    plugin_name = "openstack_gnocchi"

    profiles = ('openstack', 'openstack_controller')
    apachepkg = None

    def setup(self):
        self.add_copy_spec([
            "/etc/gnocchi/*",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/gnocchi/*",
                f"/var/log/{self.apachepkg}*/gnocchi*"
            ])
        else:
            self.add_copy_spec([
                "/var/log/gnocchi/*.log",
                f"/var/log/{self.apachepkg}*/gnocchi*.log"
            ])

        vars_all = [p in os.environ for p in [
                    'OS_USERNAME', 'OS_PASSWORD']]

        vars_any = [p in os.environ for p in [
                    'OS_TENANT_NAME', 'OS_PROJECT_NAME']]

        if not (all(vars_all) and any(vars_any)):
            self.soslog.warning("Not all environment variables set. Source "
                                "the environment file for the user intended "
                                "to connect to the OpenStack environment.")
        else:
            self.add_cmd_output([
                "gnocchi --version",
                "gnocchi status",
                "gnocchi capabilities list",
                "gnocchi archive-policy list",
                "gnocchi resource list",
                "gnocchi resource-type list"
            ])

    def postproc(self):
        config_dir = "/etc/gnocchi"
        protect_keys = ["ceph_secret",
                        "password", "memcache_secret_key"]
        connection_keys = ["url"]
        join_con_keys = "|".join(connection_keys)

        self.do_path_regex_sub(
            f"{config_dir}/*",
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.do_path_regex_sub(
            f"{config_dir}/*",
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class RedHatGnocchi(Gnocchi, RedHatPlugin):

    apachepkg = 'httpd'
    var_puppet_gen = "/var/lib/config-data/puppet-generated/gnocchi"

    packages = (
        'openstack-gnocchi-metricd', 'openstack-gnocchi-common',
        'openstack-gnocchi-statsd', 'openstack-gnocchi-api',
        'openstack-gnocchi-carbonara'
    )

    def setup(self):
        super().setup()
        self.add_copy_spec([
            self.var_puppet_gen + "/etc/gnocchi/*",
            self.var_puppet_gen + "/etc/httpd/conf/*",
            self.var_puppet_gen + "/etc/httpd/conf.d/*",
            self.var_puppet_gen + "/etc/httpd/conf.modules.d/wsgi.conf",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf"
        ])

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/gnocchi/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/gnocchi/*",
            regexp, subst
        )

    def postproc(self):
        super().postproc()
        protect_keys = ["ceph_secret",
                        "password", "memcache_secret_key"]
        connection_keys = ["url"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianGnocchi(Gnocchi, DebianPlugin, UbuntuPlugin):

    apachepkg = 'apache2'

    packages = (
        'gnocchi-api',
        'gnocchi-metricd',
        'gnocchi-common',
        'gnocchi-statsd',
        'python-gnocchi',
        'python3-gnocchi',
    )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Convert2RHEL(Plugin, RedHatPlugin):
    """This plugin collects data generated by Convert2RHEL."""

    short_desc = 'Convert2RHEL'
    plugin_name = 'convert2rhel'
    profiles = ('system',)
    packages = ('convert2rhel',)
    verify_packages = ('convert2rhel$',)

    def setup(self):

        self.add_copy_spec([
            "/var/log/convert2rhel/convert2rhel.log",
            "/var/log/convert2rhel/archive/convert2rhel-*.log",
            "/var/log/convert2rhel/rpm_va.log",
            # Convert2RHEL generates a {pre,post}-conversion report.
            "/var/log/convert2rhel/convert2rhel-*-conversion.*",
        ])


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class JujuAgentConfScrubbedTest(StageTwoReportTest):
    """Ensure that agent conf is picked up and properly scrubbed

    :avocado: tags=stagetwo
    """

    sos_cmd = '-o juju'
    ubuntu_only = True
    files = [('agent.conf', '/var/lib/juju/agents/machine-0/agent.conf')]

    def test_unit_agent_conf_collected(self):
        self.assertFileCollected('/var/lib/juju/agents/machine-0/agent.conf')

    def test_unit_agent_conf_scrubbed(self):
        # Ensure that we scrubbed all passwords/secrets and the certificates
        check_text_scrub = [
            'cR97RRZheQgiLDrnkGxp8mlE',
            '111512798b7abbd4c2884e4de83d7989',
            'FaIBoxLXcbn6RghOnbZBh0r7v8a8PTdQWJ9fG5ey6OJLjTSd/Fk8',
            'MIIEEjCCAnqgAwIBAgIUXRUjAHgO+z8hifta518W/MvxZ9gwDQYJKoZIhvcNAQEL',
            'MIIEfjCCAuagAwIBAgIVANVl9irudWew6MVfGuzX85+5nW/OMA0GCSqGSIb3DQEB',
            'MIIEEjCCAnqgAwIBAgIUXRUjAHgO+z8hifta518W/MvxZ9gwDQYJKoZIhvcNAQEL',
            'MIIG/AIBADANBgkqhkiG9w0BAQEFAASCBuYwggbiAgEAAoIBgQC86bxFnYDYFssg',
            'MIIG/gIBADANBgkqhkiG9w0BAQEFAASCBugwggbkAgEAAoIBgQCfDj2vFQlsDlV4',
            'MIIEowIBAAKCAQEAsQmlk3a4OBmBNSy43bl66+rX+5sTsu+2yO93E/iGuzmGqX0t',
            'EB+ZKAkbUOMtPnhcFnZImnvxy658IPxGxr1ZoigInbnRr13h5/g=',
            'BjEUrba1VcDLW3fJQOxTH7R7wzM1bbu2p8R2ZnfUAWNXjr+mHuMc3mCnkEFL/X+/',
            'EB+ZKAkbUOMtPnhcFnZImnvxy658IPxGxr1ZoigInbnRr13h5/g=',
            'MK5DNKHKNIynw+tXbuJ2pQ==',
            'gGizOraTiFeIuvHMD3KATTLc',
            'L/x36ewmw1rsKYlFI5X/6qM6n5DIKU+IJGNj5VSYb1u3Q7TZZ1yF',
        ]
        for text in check_text_scrub:
            self.assertFileNotHasContent(
                '/var/lib/juju/agents/machine-0/agent.conf', text)



# Copyright 2023 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import logging


class SoSPrepper():
    """
    A prepper is a way to prepare loaded mappings with selected items within
    an sos report prior to beginning the full obfuscation routine.

    This was previously handled directly within archives, however this is a bit
    cumbersome and doesn't allow for all the flexibility we could use in this
    effort.

    Preppers are separated from parsers but will leverage them in order to feed
    parser-matched strings from files highlighted by a Prepper() to the
    appropriate mapping for initial obfuscation.

    Preppers may specify their own priority in order to influence the order in
    which mappings are prepped. Further, Preppers have two ways to prepare
    the maps - either by generating a list of filenames or via directly pulling
    content out of select files without the assistance of a parser. A lower
    priority value means the prepper should run sooner than those with higher
    values.

    For the former approach, `Prepper._get_$parser_file_list()` should be used
    and should yield filenames that exist in target archives. For the latter,
    the `Prepper._get_items_for_$map()` should be used.

    Finally, a `regex_items` dict is available for storing individual regex
    items for parsers that rely on them. These items will be added after all
    files and other individual items are handled. This dict has keys set to
    parser/mapping names, and the values should be sets of items, so preppers
    should add to them like so:

        self.regex_items['hostname'].add('myhostname')
    """

    name = 'Undefined'
    priority = 100

    def __init__(self, options):
        self.regex_items = {
            'hostname': set(),
            'ip': set(),
            'ipv6': set(),
            'keyword': set(),
            'mac': set(),
            'username': set()
        }
        self.opts = options
        self.soslog = logging.getLogger('sos')
        self.ui_log = logging.getLogger('sos_ui')

    def _fmt_log_msg(self, msg):
        return f"[prepper:{self.name}] {msg}"

    def log_debug(self, msg):
        self.soslog.debug(self._fmt_log_msg(msg))

    def log_info(self, msg):
        self.soslog.info(self._fmt_log_msg(msg))

    def log_error(self, msg):
        self.soslog.error(self._fmt_log_msg(msg))

    def get_parser_file_list(self, parser, archive):
        """
        Helper that calls the appropriate Prepper method for the specified
        parser. This allows Preppers to be able to provide items for multiple
        types of parsers without needing to handle repetitious logic to
        determine which parser we're interested within each individual call.

        The convention to use is to define `_get_$parser_file_list()` methods
        within Preppers, e.g. `_get_hostname_file_list()` would be used to
        provide filenames for the hostname parser. If such a method is not
        defined within a Prepper for a given parser, we handle that here so
        that individual Preppers do not need to.

        :param parser: The _name_ of the parser to get a file list for
        :type parser:  ``str``

        :param archive: The archive we are operating on currently for the
                        specified parser
        :type archive:  ``SoSObfuscationArchive``

        :returns: A list of filenames within the archive to prep with
        :rtype: ``list``
        """
        _check = f"_get_{parser}_file_list"
        if hasattr(self, _check):
            return getattr(self, _check)(archive)
        return []

    def get_items_for_map(self, mapping, archive):
        """
        Similar to `get_parser_file_list()`, a helper for calling the specific
        method for generating items for the given `map`. This allows Preppers
        to be able to provide items for multiple types of maps, without the
        need to handle repetitious logic to determine which parser we're
        interested in within each individual call.

        :param mapping: The _name_ of the mapping to get items for
        :type mapping:  ``str``

        :param archive: The archive we are operating on currently for the
                        specified parser
        :type archive:  ``SoSObfuscationArchive``

        :returns: A list of distinct items to obfuscate without using a parser
        :rtype:   ``list``
        """
        _check = f"_get_items_for_{mapping}"
        if hasattr(self, _check):
            return getattr(self, _check)(archive)
        return []

# vim: set et ts=4 sw=4 :



# Copyright (C) 2025 Canonical Ltd.,
#                    Bryan Fraschetti <bryan.fraschetti@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Helm(Plugin, IndependentPlugin):
    """The Helm plugin collects information about the currently installed
    Helm charts, plugins, and repositories used in delpoyments
    """

    short_desc = 'The k8s templating and deployment manager'
    plugin_name = "helm"
    profiles = ('container', 'packagemanager')

    packages = ('helm',)

    helm_cmd = "helm"

    def setup(self):
        helm_subcmds = [
            'repo list',
            'plugin list',
            'list -a',
            'version',
        ]

        self.add_cmd_output([
            f"{self.helm_cmd} {subcmd}" for subcmd in helm_subcmds
        ])

# vim: set et ts=4 sw=4 :



# Copyright (c) 2022 Tencent., ZoeDong <zoedong@tencent.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.policies.distros.redhat import RedHatPolicy


class OpenCloudOSPolicy(RedHatPolicy):
    vendor = "OpenCloudOS"
    vendor_urls = [('Distribution Website', 'https://www.opencloudos.org/')]
    os_release_name = 'OpenCloudOS Stream'
    os_release_file = ''

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2023 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import re
from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Coredump(Plugin, IndependentPlugin):
    """
    This plugin collects from the systemd-coredump service, which is leveraged
    via the coredumpctl command.

    Beyond basic configuration of the service, the plugin will also attempt to
    collect information on available coredumps. For any coredumps reported by
    coredumpctl for which the core is present, this plugin will collect that
    coredump file. The number of dump files collects is controlled by the
    `dumps` option, defaulting to 3. Note that core collection is performed
    based on the most recent cores generated. The summary information provided
    by `coredumpctl info` is also collected.

    The coredump files will be at their canonical location in the sos archive,
    usually under var/lib/systemd/coredump. Symlinks are dropped in the plugin
    directory to assist in identifying which coredump aligns to which entry in
    the provided `coredumpctl list` output collection.

    Users may leverage the `executable` option to control which coredumps the
    plugin will collect. This option takes a case-insensitive python regex
    string. If provided, only coredump entries for which the EXE field in
    `coredumpctl list` output matches will be collected.

    The dump files collected are compressed, and users should be aware that
    when inflated these files can be orders of magnitude larger than their
    collected sizes.
    """

    short_desc = 'systemd-coredump related information and dump files'

    plugin_name = "coredump"
    profiles = ('system', 'debug')
    packages = ('systemd-udev', 'systemd-coredump')

    option_list = [
        PluginOpt("dumps", default=3, desc="number of dump files to collect"),
        PluginOpt("executable", default='',
                  desc=("only collect info and dump output for executables "
                        "matching this regex"))
    ]

    def setup(self):
        self.add_copy_spec([
            "/etc/systemd/coredump.conf",
            "/etc/systemd/coredump.conf.d/",
            "/run/systemd/coredump.conf.d/",
            "/usr/lib/systemd/coredump.conf.d/",
            "/usr/lib/systemd/systemd-coredump"
        ])

        coredump_list = self.collect_cmd_output("coredumpctl list --reverse")
        if coredump_list['status'] != 0:
            return

        cores_collected = 0
        for line in coredump_list["output"].splitlines()[1:]:
            cdump = line.split()
            pid = cdump[4]
            exe = cdump[-2]
            if regex := self.get_option("executable"):
                if not re.search(regex, exe, re.I):
                    continue
            cinfo = self.collect_cmd_output(f"coredumpctl info {pid}")
            if cinfo['status'] != 0:
                continue
            res = cinfo['output']
            if cores_collected < self.get_option("dumps"):
                core = re.search(r"(^\s*Storage:(.*)(\(present\)))", res, re.M)
                try:
                    core_path = core.groups()[1].strip()
                    # a_c_s does not return any information for a skipped file,
                    # so stat the size here and if the core is larger than our
                    # limit, move on to the next
                    # TODO: do not hardcode this. Extend log-size to per-plugin
                    # TODO: option and link this to that value
                    if os.stat(core_path).st_size > 209715200:
                        self._log_info(
                            f"Skipping core dump file {core_path} due to size"
                        )
                        continue
                    self.add_copy_spec(core_path, tailit=False, sizelimit=200)
                    plugpath = self.path_join(
                        self.commons['cmddir'],
                        self.name(),
                        f"coredump_{pid}_{self._mangle_command(exe)}"
                    )
                    # use os.path instead of the plugin wrapper here so that
                    # we always get the real path of the archive on disk
                    arcpath = os.path.join(self.archive.get_archive_path(),
                                           core_path)
                    linkpath = os.path.relpath(arcpath, plugpath)
                    linkpath = linkpath.replace('../', '', 1)
                    self.archive.add_link(linkpath, plugpath)
                    cores_collected += 1
                except AttributeError:
                    # no match on the re.search()
                    pass
                except Exception as err:
                    self._log_info(
                        f"Could not collect coredump for {pid} : {err}"
                    )

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Alan Baghumian <alan.baghumian@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
from sos.report.plugins import Plugin, UbuntuPlugin


class MicroOVN(Plugin, UbuntuPlugin):
    """The MicroOVN plugin collects the current status of the microovn
    snap.

    It will collect journald logs as well as output from various microovn
    commands.
    """

    short_desc = 'MicroOVN Snap'
    plugin_name = "microovn"
    profiles = ('network', 'virt')

    packages = ('microovn', )
    commands = ('microovn', )

    def setup(self):
        self.add_journal(units="snap.microovn.*")

        microovn_subcmds = [
            'cluster list',
            'status',
            'certificates list',
            '--version'
        ]
        self.add_cmd_output([
            f"microovn {subcmd}" for subcmd in microovn_subcmds
        ])

        dqlite_crt = "/var/snap/microovn/common/state/cluster.crt"
        self.add_cmd_output(
            f"openssl x509 -in {dqlite_crt} -noout -dates",
        )

        db_path = "/var/snap/microovn/common/state/database"

        # Check for inconsistent dqlite db intervals
        self.add_dir_listing(
            db_path,
            suggest_filename="ls_microovn_dqlite_dir",
        )

        self.add_copy_spec([
                f"{db_path}/info.yaml",
                f"{db_path}/cluster.yaml",
                f"{db_path}/../daemon.yaml",
        ])

        queries = [
            {
                "query": "SELECT * FROM sqlite_master WHERE type=\"table\";",
                "suggested_file_suffix": "schema",
            },
            {
                "query": (
                    "SELECT * FROM config WHERE NOT ( "
                    "key LIKE \"%keyring%\" OR "
                    "key LIKE \"%ca_cert%\" OR "
                    "key LIKE \"%ca_key%\" );"
                ),
                "suggested_file_suffix": "config",
            },
            {
                "query": "SELECT * FROM services;",
                "suggested_file_suffix": "services",
            },
            {
                "query": (
                    "SELECT id, name, expiry_date "
                    "FROM core_token_records;"
                ),
                "suggested_file_suffix": "token_records",
            },
            {
                "query": (
                    "SELECT id, name, address, schema_internal, "
                    "schema_external, heartbeat, role, api_extensions "
                    "FROM core_cluster_members;"
                ),
                "suggested_file_suffix": "core_cluster_members",
            },
        ]

        for query_entry in queries:
            query = json.dumps(query_entry.get("query"))
            file_suffix = query_entry.get("suggested_file_suffix")
            self.add_cmd_output(
                f"microovn cluster sql {query}",
                suggest_filename=f"microovn_cluster_sql_{file_suffix}",
            )

# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Cedric Jeanneret <cjeanner@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class OpenStackTripleO(Plugin, IndependentPlugin):

    short_desc = 'Installation information from OpenStack Installer'

    plugin_name = 'openstack_tripleo'
    profiles = ('openstack', 'openstack_controller', 'openstack_compute')
    packages = ('openstack-selinux',)
    tripleo_log_paths = []

    def setup(self):
        # Notes: recursion is max 2 for container-puppet and tripleo-config
        # Those directories are present on all OpenStack nodes
        self.tripleo_log_paths = [
            '/var/log/paunch.log',
            '/var/lib/container-puppet/',
            '/var/lib/tripleo-config/',
            '/var/lib/tripleo/',
            '/etc/puppet/hieradata/'
        ]
        self.add_copy_spec(self.tripleo_log_paths)

    def postproc(self):
        # Ensures we do not leak passwords from the tripleo-config and
        # hieradata locations.
        # Other locations don't have sensitive data.
        regexp = r'(".*(key|password|pass|secret|database_connection))' \
                 r'([":\s]+)(.*[^"])([",]+)'
        for path in self.tripleo_log_paths:
            self.do_path_regex_sub(path, regexp, r'\1\3*********\5')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import yaml

from sos.report.plugins import Plugin, UbuntuPlugin

SNAP_COMMON_PATH = "/var/snap/charmed-postgresql/common"
SNAP_CURRENT_PATH = "/var/snap/charmed-postgresql/current"

PATHS = {
    "POSTGRESQL_CONF": SNAP_COMMON_PATH + "/var/lib/postgresql",
    "POSTGRESQL_LOGS": SNAP_COMMON_PATH + "/var/log/postgresql",
    "PATRONI_CONF": SNAP_CURRENT_PATH + "/etc/patroni",
    "PATRONI_LOGS": SNAP_COMMON_PATH + "/var/log/patroni",
    "PGBACKREST_CONF": SNAP_CURRENT_PATH + "/etc/pgbackrest",
    "PGBACKREST_LOGS": SNAP_COMMON_PATH + "/var/log/pgbackrest",
    "PGBOUNCER_CONF": SNAP_CURRENT_PATH + "/etc/pgbouncer",
    "PGBOUNCER_LOGS": SNAP_COMMON_PATH + "/var/log/pgbouncer",
}

PATRONI_CONFIG_FILE = f"{PATHS['PATRONI_CONF']}/patroni.yaml"

RUNAS = "snap_daemon"
PSQL = "charmed-postgresql.psql"
PATRONICTL = "charmed-postgresql.patronictl"


class CharmedPostgreSQL(Plugin, UbuntuPlugin):

    short_desc = "Charmed PostgreSQL"
    plugin_name = "charmed_postgresql"
    packages = ('charmed-postgresql',)

    @property
    def patronictl_args(self) -> str:
        return f"--config-file {PATRONI_CONFIG_FILE}"

    @property
    def psql_args(self) -> str:
        return (f"-U {self.postgresql_username} "
                f"-h {self.postgresql_host} "
                f"-p {self.postgresql_port} "
                r"-d postgres -P pager=off")

    def setup(self):
        # --- FILE EXCLUSIONS ---

        # Keys and certificates
        self.add_forbidden_path([
            f"{PATHS['PATRONI_CONF']}/*.pem",
            f"{PATHS['PGBOUNCER_CONF']}/*.pem",
        ])

        # --- FILE INCLUSIONS ---

        self.add_copy_spec([
            f"{PATHS['POSTGRESQL_CONF']}/*.conf*",
            f"{PATHS['POSTGRESQL_LOGS']}",
            f"{PATHS['PATRONI_CONF']}/*.y*ml",
            f"{PATHS['PATRONI_LOGS']}",
            f"{PATHS['PGBACKREST_CONF']}",
            f"{PATHS['PGBACKREST_LOGS']}",
            f"{PATHS['PGBOUNCER_CONF']}",
            f"{PATHS['PGBOUNCER_LOGS']}",
        ])

        # --- SNAP LOGS ---

        self.add_journal("snap.charmed-postgresql.*")

        # --- SNAP INFO ---

        self.add_cmd_output(
            "snap info charmed-postgresql",
            suggest_filename="snap-info",
        )

        # Read and parse patroni config, finish setup if there are errors
        try:
            with open(PATRONI_CONFIG_FILE, encoding='utf-8') as f:
                patroni_config = yaml.safe_load(f)
            self.patroni_cluster_name = patroni_config["scope"]
            postgresql = patroni_config["postgresql"]
            host_port = postgresql["connect_address"].split(":")
            self.postgresql_host, self.postgresql_port = host_port
            authentication = postgresql["authentication"]
            superuser = authentication["superuser"]
            self.postgresql_username = superuser["username"]
            self.postgresql_password = superuser["password"]
        except (OSError, yaml.YAMLError, TypeError,
                ValueError, KeyError, AttributeError) as error:
            self._log_warn("Can not run additional commands due to "
                           "an error on opening or parsing the config file "
                           f"{PATRONI_CONFIG_FILE}: "
                           f'{error}')
            return

        # --- TOPOLOGY ---

        self.add_cmd_output(
            (f"{PATRONICTL} {self.patronictl_args} "
             f"topology {self.patroni_cluster_name}"),
            suggest_filename="patroni-topology",
            runas=RUNAS,
        )

        # --- HISTORY ---

        self.add_cmd_output(
            (f"{PATRONICTL} {self.patronictl_args} "
             f"history {self.patroni_cluster_name}"),
            suggest_filename="patroni-history",
            runas=RUNAS,
        )

        # --- DCS CONFIGS ---

        self.add_cmd_output(
            (f"{PATRONICTL} {self.patronictl_args} "
             f"show-config {self.patroni_cluster_name}"),
            suggest_filename="patroni-dcs-config",
            runas=RUNAS,
        )

        # ADD DB PASSWORD TO ENVIRONMENT
        os.environ["PGPASSWORD"] = self.postgresql_password

        # --- DATABASES ---

        self.add_cmd_output(
            (f"{PSQL} {self.psql_args} "
             r"-c '\l+'"),
            suggest_filename="postgresql-databases",
            runas=RUNAS,
        )

        # --- USERS ---

        self.add_cmd_output(
            (f"{PSQL} {self.psql_args} "
             r"-c '\duS+'"),
            suggest_filename="postgresql-users",
            runas=RUNAS,
        )

        # --- TABLES ---

        self.add_cmd_output(
            (f"{PSQL} {self.psql_args} "
             r"-c '\dtS+'"),
            suggest_filename="postgresql-tables",
            runas=RUNAS,
        )

    def postproc(self):

        # REMOVE DB PASSWORD FROM ENVIRONMENT
        if "PGPASSWORD" in os.environ:
            del os.environ["PGPASSWORD"]

        # --- SCRUB PASSWORDS ---

        # Match lines containing password: and
        # followed by anything which may be enclosed with "
        self.do_path_regex_sub(
            f"{PATHS['PATRONI_CONF']}/*",
            r'(password: )"?.*"?',
            r'\1"*********"',
        )

        # https://pgbackrest.org/configuration.html#section-repository/option-repo-s3-key
        # https://pgbackrest.org/configuration.html#section-repository/option-repo-s3-key-secret
        self.do_path_regex_sub(
            f"{PATHS['PGBACKREST_CONF']}/pgbackrest.conf",
            r'(.*s3-key.*=).*',
            r'\1*********',
        )

        # https://www.pgbouncer.org/config.html#authentication-file-format
        self.do_path_regex_sub(
            f"{PATHS['PGBOUNCER_CONF']}/pgbouncer/userlist.txt",
            r'(".*" )".*"',
            r'\1"*********"',
        )



# Copyright (C) 2013 Adam Stokes <adam.stokes@ubuntu.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import pwd
import json
import re
from sos.report.plugins import Plugin, UbuntuPlugin, PluginOpt


class Juju(Plugin, UbuntuPlugin):
    """The Juju plugin is aimed at collecting Juju-related logs,
    configurations, and controller/model state(s).

    Logs and agent configuration information (/var/log/juju and /var/lib/juju)
    is collected by default since these are useful for troubleshooting.

    The Juju state collection is disabled by default and can be enabled with
    the 'juju-state' option. Collecting Juju state is safe in theory, but it
    does act on the live controller(s)/model(s) and is therefore optional.

    The default Juju state collection collects all controllers and models that
    the 'juju-user' (default=ubuntu) has access to.

    Specific controllers or models can be collected using the 'controllers'
    and 'models' options.

    Important: the string list is whitespace delimited, not colon delimited
    (sos plugin standard). This is due to the underlying Juju CLI accepting
    specific models in the format 'controller:model' and whitespaces are not
    allowed in either controller and model names.

    Example: models="controller_a:model_x controller_b:model_y"
    """

    short_desc = 'Juju orchestration tool'

    plugin_name = 'juju'
    profiles = ('virt', 'sysmgmt',)

    # Using files instead of packages here because there is no identifying
    # package on a juju machine.
    files = ('/var/log/juju',)

    option_list = [
        PluginOpt(
            "juju-state",
            default=False,
            val_type=bool,
            desc="Include Juju state in the report",
        ),
        PluginOpt(
            "juju-user",
            default="ubuntu",
            val_type=str,
            desc="Juju client user.",
        ),
        PluginOpt(
            "controllers",
            default="",
            val_type=str,
            desc="Collect Juju state for specified controllers. Uses a \
            whitespace delimited list.",
        ),
        PluginOpt(
            "models",
            default="",
            val_type=str,
            desc="Collect Juju state for specified models. Uses a whitespace \
            delimited list.",
        ),
    ]

    agent_name = ""

    def setup(self):
        # Juju service names are not consistent through deployments,
        # so we need to use a wildcard to get the correct service names.
        for service in self.get_service_names("juju*"):
            self.add_journal(service)
            self.add_service_status(service)

        juju_agent_cmds = {
            'juju_engine_report': 'depengine',
            'juju_goroutines': 'debug/pprof/goroutine?debug=1',
            'juju_heap_profile': 'debug/pprof/heap?debug=1',
            'juju_metrics': 'metrics',
            'juju_pubsub_report': 'pubsub',
            'juju_presence_report': 'presence',
            'juju_statepool_report': 'statepool',
            'juju_statetracker_report': ('debug/pprof/juju/state/tracker?'
                                         'debug=1'),
            'juju_unit_status': 'units?action=status',
        }

        if self.path_exists("/var/lib/juju/agents"):
            for cmd, agent_cmd in juju_agent_cmds.items():
                self.add_cmd_output(
                    self._juju_agent(agent_cmd),
                    suggest_filename=cmd
                )

        # Get agent configs for each agent.
        self.add_copy_spec("/var/lib/juju/agents/*/agent.conf")

        # Get a directory listing of /var/log/juju and /var/lib/juju
        self.add_dir_listing([
            '/var/log/juju*',
            '/var/lib/juju*'
        ], recursive=True)

        if self.get_option("all_logs"):
            # /var/lib/juju used to be in the default capture moving here
            # because it usually was way to big.  However, in most cases you
            # want all logs you want this too.
            self.add_copy_spec([
                "/var/log/juju",
                "/var/lib/juju",
                "/var/lib/juju/**/.*",
            ])
            self.add_forbidden_path("/var/lib/juju/kvm")
            self.add_forbidden_path("/var/lib/juju/tools")
        else:
            # We need this because we want to collect to the limit of all
            # logs in the directory.
            self.add_copy_spec("/var/log/juju/*.log")

        # Only include the Juju state report if this plugin option is set
        if not self.get_option("juju-state"):
            return

        juju_user = self.get_option("juju-user")
        try:
            pwd.getpwnam(juju_user)
        except KeyError:
            self._log_warn(
                f'User "{juju_user}" does not exist, '
                "will not collect Juju information."
            )
            return

        if self.get_option("controllers") and self.get_option("models"):
            self._log_warn(
                "Options: controllers, models are mutually exclusive. "
                "Will not collect Juju information."
            )
            return

        controllers_json = self.collect_cmd_output(
            "juju controllers --format=json", runas=juju_user
        )
        if controllers_json["status"] == 0:
            desired_controllers = set(
                self.get_option("controllers").split(" ")
            )
            # If a controller option is supplied, use it. Otherwise, get all
            #  controllers
            if desired_controllers and desired_controllers != {""}:
                controllers = desired_controllers
            else:
                controllers = set(
                    json.loads(controllers_json["output"])[
                        "controllers"
                    ].keys()
                )
        else:
            controllers = {}

        # Specific models
        if self.get_option("models"):
            for model in self.get_option("models").split(" "):
                command = f"juju status -m {model} --format=json"
                self.add_cmd_output(command, runas=juju_user)

        # All controllers and all models OR specific controllers and all
        #  models for each
        else:
            for controller in controllers:
                models_json = self.exec_cmd(
                    f"juju models --all -c {controller} --format=json",
                    runas=juju_user,
                )
                if models_json["status"] == 0:
                    models = json.loads(models_json["output"])["models"]
                    for model in models:
                        short_name = model["short-name"]
                        command = (
                            f"juju status -m {controller}:{short_name} "
                            f"--format=json"
                        )
                        self.add_cmd_output(command, runas=juju_user)

    def _juju_agent(self, command):
        if self.agent_name == "":
            for dir_name in self.listdir("/var/lib/juju/agents"):
                if re.search('machine-*|controller-*|application-*', dir_name):
                    self.agent_name = dir_name
                    break

        return f"juju-introspect --agent={self.agent_name} {command}"

    def postproc(self):
        agents_path = "/var/lib/juju/agents/*"
        protect_keys = [
            "sharedsecret",
            "apipassword",
            "oldpassword",
            "statepassword",
        ]

        # Redact simple yaml style "key: value".
        keys_regex = fr"(^\s*({'|'.join(protect_keys)})\s*:\s*)(.*)"
        sub_regex = r"\1*********"
        self.do_path_regex_sub(agents_path, keys_regex, sub_regex)
        # Redact certificates
        self.do_file_private_sub(agents_path)
        self.do_cmd_private_sub('juju controllers')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class NSS(Plugin, IndependentPlugin):

    short_desc = 'Network Security Services configuration'

    plugin_name = "nss"
    profiles = ('network', 'security')
    packages = ('nss',)
    verify_packages = ('nss.*',)

    def setup(self):
        self.add_forbidden_path([
            "/etc/pki/nssdb/cert*",
            "/etc/pki/nssdb/key*",
            "/etc/pki/nssdb/secmod.db"
        ])

        self.add_copy_spec("/etc/pki/nssdb/pkcs11.txt")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2023 Nvidia Corporation, Alin Serdean <aserdean@nvidia.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import time
from sos.report.plugins import Plugin, IndependentPlugin


class MellanoxFirmware(Plugin, IndependentPlugin):

    short_desc = 'Nvidia(Mellanox) firmware tools output'

    plugin_name = "mellanox_firmware"
    profiles = ('hardware', 'system')
    packages = ('mst', 'mstflint')

    MLNX_STRING = "Mellanox Technologies"

    def check_enabled(self):
        """
        Checks if this plugin should be executed at all.
        We will only enable the plugin if there is a
        Mellanox Technologies network adapter
        """
        lspci = self.exec_cmd("lspci -D -d 15b3::0200")
        return lspci['status'] == 0 and self.MLNX_STRING in lspci['output']

    def collect(self):
        if not self.get_option('allow_system_changes'):
            self._log_info("Skipping mst/mlx cable commands as system changes"
                           "would be made. Use --allow-system-changes to"
                           "enable this collection.")
            return

        # Run only if mft package is installed.
        # flint is available from the mft package.
        cout = self.exec_cmd('flint --version')
        if cout['status'] != 0:
            return

        cout = self.collect_cmd_output('mst start')
        if cout['status'] != 0:
            return

        self.collect_cmd_output('mst cable add')
        self.collect_cmd_output("mst status -v", timeout=10)
        self.collect_cmd_output("mlxcables", timeout=10)
        cout = os.listdir("/dev/mst")
        mlxcables = []
        for device in cout:
            if 'cable' in device:
                mlxcables.append(device)
        for mlxcable in mlxcables:
            self.collect_cmd_output(f"mlxcables -d {mlxcable} --DDM",
                                    timeout=10)
            self.collect_cmd_output(f"mlxcables -d {mlxcable} --dump",
                                    timeout=10)
        self.collect_cmd_output("mst stop", changes=True)

    def setup(self):
        # Get all devices which have the vendor Mellanox Technologies
        devices = []
        device_list = self.collect_cmd_output('lspci -D -d 15b3::0200')
        # Will return a string of the following format:
        # 0000:08:00.0 Ethernet controller: Mellanox Technologies MT2892 Family
        if device_list['status'] != 0:
            # bail out if there no Mellanox PCI devices
            return

        for line in device_list["output"].splitlines():
            # Should return 0000:08:00.0
            # from the following string
            # 0000:08:00.0 Ethernet controller: Mellanox Technologies MT2892
            # Family
            devices.append(line[0:8]+'00.0')

        devices = set(devices)

        # Mft package is present if OFED is installed
        # mstflint package is part of the distro and can be installed.
        commands = []

        # mft package is installed if flint command is available
        cout = self.exec_cmd('flint --version')
        if cout['status'] != 0:
            # mstflint package commands
            # the commands do not support position independent arguments
            commands = [
                ["mstconfig -d ", " -e q"],
                ["mstflint -d ", " dc"],
                ["mstflint -d ", " q"],
                ["mstreg -d ", " --reg_name ROCE_ACCL --get"],
                ["mstlink -d ", ""],
            ]
        else:
            # mft package commands
            # the commands do not support position independent arguments
            commands = [
                ["mlxdump -d ", " pcie_uc --all"],
                ["mstconfig -d ", " -e q"],
                ["flint -d ", " dc"],
                ["flint -d ", " q"],
                ["mlxreg -d ", " --reg_name ROCE_ACCL --get"],
                ["mlxlink -d ", ""],
                ["fwtrace -d ", " -i all --tracer_mode FIFO"],
            ]
        for device in devices:
            for command in commands:
                self.add_cmd_output(f"{command[0]} {device} "
                                    f"{command[1]}", timeout=30)

            # Dump the output of the mstdump command three times
            # waiting for one second. This output is useful to check
            # if certain registers changed
            for _ in range(3):
                self.add_cmd_output(f"mstdump {device}")
                time.sleep(1)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Keyutils(Plugin, RedHatPlugin):

    short_desc = 'Kernel key ring'

    plugin_name = 'keyutils'
    profiles = ('system', 'kernel', 'security', 'storage')

    packages = ('keyutils',)

    def setup(self):
        self.add_copy_spec([
            "/etc/request-key.conf",
            "/etc/request-key.d",
            "/proc/key-users"
        ])
        self.add_cmd_output("keyctl show")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Flatpak(Plugin, IndependentPlugin):

    short_desc = 'Flatpak'

    plugin_name = 'flatpak'
    profiles = ('sysmgmt', 'packagemanager')
    commands = ("flatpak",)
    packages = ("flatpak",)

    def setup(self):
        env = {"GVFS_REMOTE_VOLUME_MONITOR_IGNORE": "1"}
        self.add_cmd_output([
            "flatpak --version",
            "flatpak --default-arch",
            "flatpak --supported-arches",
            "flatpak --gl-drivers",
            "flatpak --installations",
            "flatpak --print-updated-env",
            "flatpak config",
            "flatpak remote-list --show-details",
            "flatpak list --runtime --show-details",
            "flatpak list --app --show-details",
            "flatpak history --columns=all",
        ], env=env)
        if self.get_option("verify"):
            self.add_cmd_output("flatpak repair --dry-run", env=env)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class DockerDistribution(Plugin):

    short_desc = 'Docker Distribution'
    plugin_name = "docker_distribution"
    profiles = ('container',)

    def setup(self):
        self.add_copy_spec('/etc/docker-distribution/')
        self.add_journal('docker-distribution')
        conf = self.path_join('/etc/docker-distribution/registry/config.yml')
        if self.path_exists(conf):
            with open(conf, encoding='UTF-8') as file:
                for line in file:
                    if 'rootdirectory' in line:
                        loc = line.split()[1]
                        self.add_dir_listing(loc, tree=True)


class RedHatDockerDistribution(DockerDistribution, RedHatPlugin):

    packages = ('docker-distribution',)

    def setup(self):
        self.add_forbidden_path('/etc/docker-distribution/registry/*passwd')
        super().setup()



# Copyright (C) 2008 EMC Corporation. Keith Kearnan <kearnan_keith@emc.com>
# Copyright (C) 2014 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt
from sos.utilities import is_executable


class Navicli(Plugin, RedHatPlugin):

    short_desc = 'EMC Navicli'

    plugin_name = 'navicli'
    profiles = ('storage', 'hardware')
    option_list = [
        PluginOpt('ipaddrs', default='', val_type=str,
                  desc='space-delimited list of CLARiiON IP addresses')
    ]

    def check_enabled(self):
        return is_executable("navicli")

    def get_navicli_config(self):
        """ EMC Navisphere Host Agent NAVICLI specific information - files
        """
        self.add_copy_spec([
            "/etc/Navisphere/agent.config",
            "/etc/Navisphere/Navimon.cfg",
            "/etc/Navisphere/Quietmode.cfg",
            "/etc/Navisphere/messages/[a-z]*",
            "/etc/Navisphere/log/[a-z]*"
        ])

    def get_navicli_sp_info(self, sp_address):
        """ EMC Navisphere Host Agent NAVICLI specific
        information - CLARiiON - commands
        """
        self.add_cmd_output([
            f"navicli -h {sp_address} getall",
            f"navicli -h {sp_address} getsptime -spa",
            f"navicli -h {sp_address} getsptime -spb",
            f"navicli -h {sp_address} getlog",
            f"navicli -h {sp_address} getdisk",
            f"navicli -h {sp_address} getcache",
            f"navicli -h {sp_address} getlun",
            f"navicli -h {sp_address} getlun -rg -type -default -owner -crus "
            "-capacity",
            f"navicli -h {sp_address} lunmapinfo",
            f"navicli -h {sp_address} getcrus",
            f"navicli -h {sp_address} port -list -all",
            f"navicli -h {sp_address} storagegroup -list",
            f"navicli -h {sp_address} spportspeed -get",
        ])

    def setup(self):
        self.get_navicli_config()
        for addr in set(self.get_option("ipaddrs").split()):
            if self.exec_cmd(f"navicli -h {addr} getsptime")['status'] == 0:
                self.get_navicli_sp_info(addr)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Aide(Plugin):

    short_desc = 'Advanced Intrusion Detection Environment'

    plugin_name = "aide"
    profiles = ('system', 'security')

    packages = ('aide',)
    conf_file = "/etc/aide/aide.conf"

    def setup(self):
        self.add_cmd_output(f"aide -c {self.conf_file} --config-check")

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/aide/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/aide/aide.log"
            ])


class RedHatAide(Aide, RedHatPlugin):
    conf_file = "/etc/aide.conf"

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/aide.conf",
            "/etc/logrotate.d/aide"
        ])


class DebianAide(Aide, DebianPlugin, UbuntuPlugin):
    conf_file = "/etc/aide/aide.conf"

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/aide/",
            "/etc/default/aide"
        ])

# vim: et ts=4 sw=4



# Copyright (C) 2013 Adam Stokes <adam.stokes@ubuntu.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, UbuntuPlugin, RedHatPlugin


class Azure(Plugin, UbuntuPlugin):

    short_desc = 'Microsoft Azure client'

    plugin_name = 'azure'
    profiles = ('virt',)
    packages = ('WALinuxAgent',)

    def setup(self):
        self.add_copy_spec([
            "/var/log/waagent*",
            "/var/lib/cloud",
            "/etc/default/kv-kvp-daemon-init",
            "/etc/waagent.conf",
            "/sys/module/hv_netvsc/parameters/ring_size",
            "/sys/module/hv_storvsc/parameters/storvsc_ringbuffer_size",
            "/var/lib/AzureEnhancedMonitor"
        ])

        # Adds all files under /var/log/azure to the sos report
        # os.walk is used because /var/log/azure is used by multiple Azure
        # extensions and there is no standard log filename format
        limit = self.get_option("log_size")

        for path, _, files in os.walk("/var/log/azure"):
            for name in files:
                self.add_copy_spec(self.path_join(path, name), sizelimit=limit)

        self.add_cmd_output((
            'curl -s -H Metadata:true --noproxy "*" '
            '"http://169.254.169.254/metadata/instance/compute?'
            'api-version=2023-07-01&format=json"'
        ), suggest_filename='instance_metadata.json')


class RedHatAzure(Azure, RedHatPlugin):

    def setup(self):
        super().setup()

        if self.path_isfile('/etc/yum.repos.d/rh-cloud.repo'):
            curl_cmd = ('curl -s -m 5 -vvv '
                        'https://rhui-%s.microsoft.com/pulp/repos/%s')
            self.add_cmd_output([
                curl_cmd % ('1', 'microsoft-azure-rhel7'),
                curl_cmd % ('2', 'microsoft-azure-rhel7'),
                curl_cmd % ('3', 'microsoft-azure-rhel7')
            ])

        crt_path = '/etc/pki/rhui/product/content.crt'
        if self.path_isfile(crt_path):
            self.add_cmd_output([
                'openssl x509 -noout -text -in ' + crt_path
            ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class OpenStackMasakari(Plugin, UbuntuPlugin):

    short_desc = 'OpenStack Masakari'
    plugin_name = "openstack_masakari"
    profiles = ('openstack', 'openstack_controller')

    packages = (
        'masakari-engine',
        'masakari-api',
        'python3-masakari',
    )

    services = ('masakari-engine', )

    config_dir = "/etc/masakari"

    def setup(self):

        masakari_cmd = "masakari-manage --config-dir"\
                       f"{self.config_dir} db version"
        self.add_cmd_output(
            masakari_cmd,
            suggest_filename="masakari_db_version"
        )

        self.add_copy_spec([
            self.config_dir,
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/masakari/*",
                "/var/log/apache2/masakari*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/masakari/*.log",
                "/var/log/apache2/masakari*.log",
            ])

        self.add_file_tags({
            f"{self.config_dir}/masakari.conf": "masakari_conf"
        })

    def postproc(self):
        protect_keys = [".*password.*", "transport_url",
                        "memcache_secret_key", "rabbit_password"]
        connection_keys = ["connection", "sql_connection"]

        join_con_keys = "|".join(connection_keys)

        self.do_path_regex_sub(
            f"{self.config_dir}/*",
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.do_path_regex_sub(
            f"{self.config_dir}/*",
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageTwoReportTest, redhat_only, debian_only


class Krb5PluginTest(StageTwoReportTest):
    """Ensure that the krb5 plugin activates for the distros that we support it
    on.

    See https://github.com/sosreport/sos/issues/3041

    :avocado: tags=stageone
    """

    sos_cmd = '-o krb5'
    packages = {
        'rhel': ['krb5-libs', 'krb5-server'],
        'Ubuntu': ['krb5-user', 'krb5-kdc'],
        'debian': ['krb5-user', 'krb5-kdc'],
    }

    def test_plugin_ran(self):
        self.assertPluginIncluded('krb5')

    def test_conf_collected(self):
        self.assertFileCollected('/etc/krb5.conf')

    @debian_only
    def test_ubuntu_kdcdir_collected(self):
        self.assertFileGlobInArchive('/var/lib/krb5kdc/*')

    @redhat_only
    def test_redhat_kdcdir_collected(self):
        self.assertFileGlobInArchive('/var/kerberos/krb5kdc/*')



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class VMWare(Plugin, IndependentPlugin):

    short_desc = 'VMWare client information'

    plugin_name = 'vmware'
    profiles = ('virt',)
    packages = ('open-vm-tools', 'VMWare-Tools')
    files = ('/etc/vmware-tools', '/etc/vmware')
    commands = ('vmware-toolbox-cmd',)
    services = ('vmtoolsd',)

    def setup(self):
        self.add_copy_spec([
            "/etc/vmware-tools/",
            "/etc/vmware/locations",
            "/etc/vmware/config",
            "/proc/vmmemctl",
            "/sys/kernel/debug/vmmemctl",
            "/var/log/vmware-network.log",
            "/var/log/vmware-vgauthsvc.log.0",
            "/var/log/vmware-vmsvc-root.log",
            "/var/log/vmware-vmtoolsd-root.log",
            "/var/log/vmware-vmusr-root.log"
        ])

        self.add_file_tags({
            "/etc/vmware-tools/tools.conf": "vmware_tools_conf"
        })

        self.add_cmd_output([
            "vmware-checkvm",
            "vmware-toolbox-cmd device list",
            "vmware-toolbox-cmd -v",
            "vmware-toolbox-cmd timesync status"
        ])

        stats = self.exec_cmd("vmware-toolbox-cmd stat raw")
        if stats['status'] == 0:
            for _stat in stats['output'].splitlines():
                self.add_cmd_output("vmware-toolbox-cmd stat raw text"
                                    f" {_stat}")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Qaucli(Plugin, IndependentPlugin):

    short_desc = 'QLogic information'

    plugin_name = 'qaucli'
    profiles = ('system', 'storage', 'hardware',)
    packages = ('QConvergeConsoleCLI',)

    def setup(self):
        cmd = "/opt/QLogic_Corporation/QConvergeConsoleCLI/qaucli"
        subcmds = ['-c', '-g', '-pr fc -z', '-t']

        for subcmd in subcmds:
            self.add_cmd_output(
                f"{cmd} {subcmd}",
                suggest_filename=f"qaucli_{subcmd}")

        result = self.collect_cmd_output(
                     f"{cmd} -i",
                     suggest_filename="qaucli_-i")

        if result['status'] == 0:
            for line in result['output'].splitlines():
                if "HBA Instance" in line:
                    hba = line.split(':')[1].strip()
                    self.add_cmd_output(
                        f"{cmd} -l {hba}",
                        suggest_filename=f"qaucli_-l_{hba}")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Robbie Harwood <rharwood@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class GSSProxy(Plugin, IndependentPlugin):

    short_desc = 'GSSAPI Proxy'

    plugin_name = "gssproxy"
    profiles = ('services', 'security', 'identity')
    packages = ('gssproxy',)

    def setup(self):
        self.add_copy_spec([
            "/etc/gssproxy/*.conf",
            "/etc/gss/mech.d/*"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import Plugin, IndependentPlugin


class SAS3ircu(Plugin, IndependentPlugin):
    """
    The sas3ircu plugin is intended to gather information for sas adapters,
    particularly sas-3 RAID adapters, and will collect information on each
    adapter discovered on the system.
    """

    short_desc = 'SAS-3 Integrated RAID adapter information'

    plugin_name = "sas3ircu"
    commands = ("sas3ircu",)

    def setup(self):

        # get list of adapters
        result = self.collect_cmd_output("sas3ircu list", timeout=5)

        if result["status"] == 0:
            # only want devices
            sas_lst = result["output"].splitlines()[10:-1]

            # for each adapter get some basic info
            for sas_info in sas_lst:
                sas_num = sas_info.split()[0]
                self.add_cmd_output(f"sas3ircu {sas_num} display", timeout=5)
                self.add_cmd_output(f"sas3ircu {sas_num} status", timeout=5)

# vim: et ts=4 sw=4



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Ruby(Plugin, IndependentPlugin):

    short_desc = 'Ruby runtime'

    plugin_name = 'ruby'
    packages = ('ruby', 'ruby-irb')

    def setup(self):
        self.add_cmd_output([
            'ruby --version',
            'irb --version',
            'gem --version',
            'gem list'
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Acpid(Plugin):

    short_desc = 'ACPI daemon information'
    plugin_name = "acpid"
    profiles = ('hardware',)
    packages = ('acpid',)


class RedHatAcpid(Acpid, RedHatPlugin):
    def setup(self):
        self.add_copy_spec([
            "/var/log/acpid*",
            "/etc/acpi/events/power.conf"])


class DebianAcpid(Acpid, DebianPlugin, UbuntuPlugin):
    def setup(self):
        self.add_copy_spec([
            "/etc/acpi/events/powerbtn*"])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import OpenEulerPlugin
from sos.policies.distros.redhat import RedHatPolicy


class OpenEulerPolicy(RedHatPolicy):
    vendor = "The openEuler Project"
    vendor_urls = [('Distribution Website', 'https://openeuler.org/')]
    os_release_name = 'openEuler'
    os_release_file = ''

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

        self.valid_subclasses += [OpenEulerPlugin]

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc., Pep Turro Mauri <pep@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os.path
from sos.report.plugins import Plugin, RedHatPlugin

# This plugin collects static configuration and runtime information
# about OpenShift Origin based environments, like OpenShift Enterprise 3

# Some clarification on naming:
# OpenShift Origin is the upstream project for OpenShift Enterprise,
# OpenShift Container Platflorm, and Atomic Platform.
#
# However, the name "OpenShift Origin" refers to two different code bases:
#  * Origin M5 and later (https://github.com/openshift/origin)
#    which is upstream for OpenShift 3.x and later.
#    This is what this plugin handles
#  * Origin M4 and earlier (https://github.com/openshift/origin-server)
#    which is upstream for OpenShift 1.x and 2.x.
#    This is handled by the plugin in openshift.py

# Note that this plugin should be used in conjunction with other plugins
# in order to capture relevant data: the Kubernetes plugin for the
# masters, the Docker plugin for the nodes, and also generic
# plugins (e.g. for /etc/sysconfig entries, network setup etc)


class OpenShiftOrigin(Plugin):

    short_desc = 'OpenShift Origin'

    plugin_name = "origin"
    files = None  # file lists assigned after path setup below
    profiles = ('openshift',)

    master_base_dir = "/etc/origin/master"
    node_base_dir = "/etc/origin/node"
    master_cfg = os.path.join(master_base_dir, "master-config.yaml")
    master_env = os.path.join(master_base_dir, "master.env")
    node_cfg_file = "node-config.yaml"
    node_cfg = os.path.join(node_base_dir, node_cfg_file)
    node_kubeconfig = os.path.join(node_base_dir, "node.kubeconfig")
    static_pod_dir = os.path.join(node_base_dir, "pods")

    files = (master_cfg, node_cfg)

    # Master vs. node
    #
    # OpenShift Origin/3.x cluster members can be a master, a node, or both at
    # the same time: in most deployments masters are also nodes in order to get
    # access to the pod network, which some functionality (e.g. the API proxy)
    # requires. Therefore the following methods may all evaluate True on a
    # single instance (at least one must evaluate True if this is an OpenShift
    # installation)
    def is_master(self):
        """Determine if we are on a master"""
        return self.path_exists(self.master_cfg)

    def is_node(self):
        """Determine if we are on a node"""
        return self.path_exists(self.node_cfg)

    def is_static_etcd(self):
        """Determine if we are on a node running etcd"""
        return self.path_exists(self.path_join(self.static_pod_dir,
                                               "etcd.yaml"))

    def is_static_pod_compatible(self):
        """Determine if a node is running static pods"""
        return self.path_exists(self.static_pod_dir)

    def setup(self):
        bstrap_node_cfg = self.path_join(self.node_base_dir,
                                         "bootstrap-" + self.node_cfg_file)
        bstrap_kubeconfig = self.path_join(self.node_base_dir,
                                           "bootstrap.kubeconfig")
        node_certs = self.path_join(self.node_base_dir, "certs", "*")
        node_client_ca = self.path_join(self.node_base_dir, "client-ca.crt")
        admin_cfg = self.path_join(self.master_base_dir, "admin.kubeconfig")
        oc_cmd_admin = f"oc --config={admin_cfg}"
        static_pod_logs_cmd = "master-logs"

        # Note that a system can run both a master and a node.
        # See "Master vs. node" above.
        if self.is_master():
            self.add_copy_spec([
                self.master_cfg,
                self.master_env,
                self.path_join(self.master_base_dir, "*.crt"),
            ])

            if self.is_static_pod_compatible():
                self.add_copy_spec(self.path_join(self.static_pod_dir,
                                                  "*.yaml"))
                self.add_cmd_output([
                    f"{static_pod_logs_cmd} api api",
                    f"{static_pod_logs_cmd} controllers controllers",
                ])

            if self.is_static_etcd():
                self.add_cmd_output(f"{static_pod_logs_cmd} etcd etcd")

            # Possible enhancements:
            # some thoughts about information that might also be useful
            # to collect. However, these are maybe not needed in general
            # and/or present some challenges (scale, sensitive, ...) and need
            # some more thought. For now just leaving this comment here until
            # we decide if it's worth collecting:
            #
            # General project status:
            #   oc status --all-namespaces   (introduced in OSE 3.2)
            #      -> deemed as not worthy in BZ#1394527
            # Metrics deployment configurations
            #   oc get -o json dc -n openshift-infra
            # Logging stack deployment configurations
            #   oc get -o json dc -n logging
            #
            # Note: Information about nodes, events, pods, and services
            # is already collected by the Kubernetes plugin

            subcmds = [
                "adm top images",
                "adm top imagestreams",
                "adm top nodes"
            ]

            self.add_cmd_output([
                f'{oc_cmd_admin} {subcmd}' for subcmd in subcmds
            ])

            jcmds = [
                "hostsubnet",
                "clusternetwork",
                "netnamespaces"
            ]

            self.add_cmd_output([
                f'{oc_cmd_admin} get -o json {jcmd}' for jcmd in jcmds
            ])

            nmsps = [
                'default',
                'openshift-web-console',
                'openshift-ansible-service-broker',
                'openshift-sdn',
                'openshift-console'
            ]

            self.add_cmd_output([
                f'{oc_cmd_admin} get -o json deploymentconfig,deployment,'
                f'daemonsets -n {n}' for n in nmsps
            ])

            if not self.is_static_pod_compatible():
                self.add_journal(units=["atomic-openshift-master",
                                        "atomic-openshift-master-api",
                                        "atomic-openshift-master-controllers"])

            # get logs from the infrastructure pods running in the default ns
            pods = self.exec_cmd(f"{oc_cmd_admin} get pod -o name -n default")
            for pod in pods['output'].splitlines():
                self.add_cmd_output(f"{oc_cmd_admin} logs -n default {pod}")

        # Note that a system can run both a master and a node.
        # See "Master vs. node" above.
        if self.is_node():
            self.add_copy_spec([
                self.node_cfg,
                self.node_kubeconfig,
                node_certs,
                node_client_ca,
                bstrap_node_cfg,
                bstrap_kubeconfig,
                self.path_join(self.node_base_dir, "*.crt"),
                self.path_join(self.node_base_dir, "resolv.conf"),
                self.path_join(self.node_base_dir, "node-dnsmasq.conf"),
            ])

            self.add_journal(units="atomic-openshift-node")

    def postproc(self):
        # Clear env values from objects that can contain sensitive data
        # Sample JSON content:
        #           {
        #              "name": "MYSQL_PASSWORD",
        #              "value": "mypassword"
        #           },
        # This will mask values when the "name" looks susceptible of
        # values worth obfuscating, i.e. if the name contains strings
        # like "pass", "pwd", "key" or "token".
        env_regexp = r'(?P<var>{\s*"name":\s*[^,]*' \
                     r'(pass|pwd|key|token|cred|secret' \
                     r'|PASS|PWD|KEY|TOKEN|CRED|SECRET)[^,]*,' \
                     r'\s*"value":)[^}]*'
        self.do_cmd_output_sub('oc*json', env_regexp, r'\g<var> "********"')
        # LDAP identity provider (bindPassword)
        # and github/google/OpenID identity providers (clientSecret)
        self.do_file_sub(self.master_cfg,
                         r"(bindPassword|clientSecret):\s*(.*)",
                         r'\1:"********"')


class AtomicOpenShift(OpenShiftOrigin, RedHatPlugin):

    short_desc = 'OpenShift Enterprise / OpenShift Container Platform'

    packages = ('atomic-openshift',)

# vim: set et ts=4 sw=4 :



#!/usr/bin/env python

from setuptools import setup, find_packages
from sos import __version__ as VERSION
from sos.policies import load

data_files = [
    ('share/man/man1', ['man/en/sos-report.1', 'man/en/sos.1',
                        'man/en/sos-collect.1', 'man/en/sos-clean.1',
                        'man/en/sos-mask.1', 'man/en/sos-help.1',
                        'man/en/sos-upload.1']),
    ('share/man/man5', ['man/en/sos.conf.5']),
    ('config', ['sos.conf', 'tmpfiles/tmpfilesd-sos-rh.conf'])
]

if load().os_release_name not in ("Ubuntu", "Debian"):
    data_files.extend([
        ('share/licenses/sos', ['LICENSE']),
        ('share/doc/sos', ['AUTHORS', 'README.md'])
    ])

setup(
    name='sos',
    version=VERSION,
    # to avoid a packaging dependency on older RHELs
    # we only declare it on recent Python versions
    install_requires=['pexpect', 'pyyaml', 'packaging;python_version>="3.11"'],
    description=(
        'A set of tools to gather troubleshooting information from a system'
    ),
    author='Bryn M. Reeves',
    author_email='bmr@redhat.com',
    maintainer='Jake Hunsaker',
    maintainer_email='jacob.r.hunsaker@gmail.com',
    url='https://github.com/sosreport/sos',
    license="GPLv2+",
    scripts=['bin/sos'],
    data_files=data_files,
    packages=find_packages(include=['sos', 'sos.*'])
)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Anacron(Plugin, IndependentPlugin):

    short_desc = 'Anacron job scheduling service'

    plugin_name = 'anacron'
    profiles = ('system',)
    packages = ('anacron', 'chronie-anacron')

    # anacron may be provided by anacron, cronie-anacron etc.
    # just look for the configuration file which is common
    files = ('/etc/anacrontab',)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class TPM2(Plugin, IndependentPlugin):
    """
    Collects information about TPM2 module installed on host system.

    This plugin will capture data using tpm2_tools
    """

    short_desc = 'Trusted Platform Module 2.0'
    plugin_name = 'tpm2'
    profiles = ('security', 'system', 'storage', 'hardware')
    packages = ('tpm2-tools',)

    def setup(self):
        self.add_cmd_output([
            'tpm2_getcap properties-variable',
            'tpm2_getcap properties-fixed',
            'tpm2_nvreadpublic',
            'tpm2_readclock'
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Frr(Plugin, RedHatPlugin):
    """
    FRR is a routing project that provides numerous traditional routing
    protocols for Linux platforms. In particular, OpenStack uses FRR to provide
    BGP functionality for the overcloud nodes.

    This plugin is primarily designed the deployment of FRR within OSP
    environments, which deploy FRR in a container.
    """

    short_desc = 'Frr routing service'

    plugin_name = 'frr'
    profiles = ('network',)

    files = ('/etc/frr/zebra.conf',)
    packages = ('frr',)
    containers = ('frr',)

    def setup(self):
        var_ansible_gen = "/var/lib/config-data/ansible-generated/frr"
        self.add_copy_spec([
            "/etc/frr/",
            var_ansible_gen + "/etc/frr/",
        ])

        if self.container_exists('frr'):
            subcmds = [
                'show bgp detail',
                'show bgp neighbors',
                'show bgp summary',
                'show history',
                'show ip bgp detail',
                'show ip bgp neighbors',
                'show ip bgp summary',
                'show ip bgp',
                'show ip route',
                'show ipv6 route',
                'show running-config',
                'show version',
            ]

            self.add_cmd_output(
                [f"vtysh -c '{subcmd}'" for subcmd in subcmds],
                container='frr'
            )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
from sos.report.plugins import Plugin, IndependentPlugin


class Libvirt(Plugin, IndependentPlugin):

    short_desc = 'libvirt virtualization API'

    plugin_name = 'libvirt'
    profiles = ('system', 'virt', 'openstack_edpm')

    def setup(self):
        libvirt_keytab = "/etc/libvirt/krb5.tab"

        # authentication databases used for libvirt SASL authentication
        self.add_forbidden_path([
            "/etc/libvirt/passwd.db",
            "/etc/libvirt/krb5.tab",
            "/var/lib/libvirt/qemu/*/master-key.aes",
            "/etc/libvirt/secrets"
        ])

        self.add_copy_spec([
            "/etc/libvirt/libvirt.conf",
            "/etc/libvirt/libvirtd.conf",
            "/etc/libvirt/lxc.conf",
            "/etc/libvirt/nwfilter/*.xml",
            "/etc/libvirt/qemu/*.xml",
            "/etc/libvirt/qemu.conf",
            "/run/libvirt/",
            "/etc/libvirt/qemu/networks/*.xml",
            "/etc/libvirt/qemu/networks/autostart/*.xml",
            "/etc/libvirt/storage/*.xml",
            "/etc/libvirt/storage/autostart/*.xml",
            "/etc/libvirt/qemu-lockd.conf",
            "/etc/libvirt/virtlockd.conf",
            "/etc/libvirt/virtlogd.conf",
            "/var/lib/libvirt/dnsmasq/*",
            "/var/lib/libvirt/qemu/snapshot/*/*.xml",
            "/var/lib/openstack/config/libvirt",
            "/var/lib/openstack/containers/libvirt*.json",
        ])

        if not self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/libvirt/libvirtd.log",
                "/var/log/libvirt/qemu/*.log*",
                "/var/log/libvirt/lxc/*.log",
                "/var/log/libvirt/uml/*.log",
                "/var/log/swtpm/libvirt/qemu/*.log",
                "/var/log/containers/libvirt/libvirtd.log",
                "/var/log/containers/libvirt/qemu/*.log*",
                "/var/log/containers/libvirt/lxc/*.log",
                "/var/log/containers/libvirt/swtpm/libvirt/qemu/*.log",
                "/var/log/containers/libvirt/uml/*.log",
                "/var/log/containers/qemu/*.log",
                "/var/log/containers/libvirt/*.log",
            ])
        else:
            self.add_copy_spec([
                "/var/log/libvirt",
                "/var/log/containers/qemu/",
                "/var/log/containers/libvirt/",
            ])

        if self.path_exists(self.path_join(libvirt_keytab)):
            self.add_cmd_output(f"klist -ket {libvirt_keytab}")

        self.add_dir_listing("/var/lib/libvirt/qemu", recursive=True)

        # get details of processes of KVM hosts
        for pidfile in glob.glob("/run/libvirt/*/*.pid"):
            with open(pidfile, 'r', encoding='UTF-8') as pfile:
                pid = pfile.read().splitlines()[0]
                pr_files = ["environ", "cgroup", "maps", "numa_maps", "limits"]
                for file in pr_files:
                    self.add_copy_spec(f"/proc/{pid}/{file}")

        self.add_file_tags({
            "/run/libvirt/qemu/*.xml": "var_qemu_xml",
            "/var/log/libvirt/qemu/*.log": "libvirtd_qemu_log"
        })

    def postproc(self):
        match_exp = r"(\s*passwd=\s*')([^']*)('.*)"
        libvirt_path_exps = [
            r"/etc/libvirt/qemu/.*\.xml",
            r"/run/libvirt/qemu/.*\.xml",
            r"/etc/libvirt/.*\.conf"
        ]
        for path_exp in libvirt_path_exps:
            self.do_path_regex_sub(path_exp, match_exp, r"\1******\3")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc.
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class RhvAnalyzer(Plugin, RedHatPlugin):

    short_desc = 'RHV Log Collector Analyzer'

    packages = ('rhv-log-collector-analyzer',)

    plugin_name = 'rhv_analyzer'
    profiles = ('virt',)

    def setup(self):
        tool_name = 'rhv-log-collector-analyzer'
        report = f"{self.get_cmd_output_path()}/analyzer-report.html"

        self.add_cmd_output([
            f"{tool_name} --live --html={report}",
            f"{tool_name} --json"
        ])

# vim: expandtab tabstop=4 shiftwidth=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest

DO_SKIP = '/tmp/sos-test-version.txt'
NO_SKIP = '/tmp/sos-test-version-noskip'


class SkipVersionIPParser(StageTwoReportTest):
    """Ensures that we _skip_ files ending in 'version' (or 'version.txt') to
    avoid incorrectly obfuscating version numbers.

    :avocado: tags=stagetwo
    """

    files = [
        ('sos-test-version.txt', DO_SKIP),
        ('sos-test-version-noskip', NO_SKIP)
    ]
    install_plugins = ['skip_versions']
    sos_cmd = '--clean -o skip_versions'

    def test_version_file_skipped(self):
        self.assertFileCollected(DO_SKIP)
        self.assertFileHasContent(DO_SKIP, '10.11.12.13')
        self.assertFileHasContent(DO_SKIP, '6.0.0.1')

    def test_incorrect_version_file_not_skipped(self):
        self.assertFileCollected(NO_SKIP)
        self.assertFileNotHasContent(NO_SKIP, '10.11.12.13')
        self.assertFileNotHasContent(NO_SKIP, '6.0.0.1')
